@@ -3,9 +3,10 @@ import datetime
 import io
 
 import aiofiles
-import falcon
 import PIL.Image
 
+import falcon
+
 
 class Image:
     def __init__(self, config, image_id, size):
@@ -13,7 +14,7 @@ class Image:
 
         self.image_id = image_id
         self.size = size
-        self.modified = datetime.datetime.utcnow()
+        self.modified = datetime.datetime.now(datetime.timezone.utc)
 
     @property
     def path(self):
