@@ -1,10 +1,10 @@
 import os
 import subprocess
+import sys
 import time
 
 import requests
 
-
 LOOK_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
 
 gunicorn = None
@@ -14,7 +14,14 @@ def pytest_sessionstart(session):
     global gunicorn
 
     gunicorn = subprocess.Popen(
-        ('gunicorn', '--pythonpath', LOOK_PATH, 'look.app:get_app()'),
+        (
+            sys.executable,
+            '-m',
+            'gunicorn',
+            '--pythonpath',
+            LOOK_PATH,
+            'look.app:get_app()',
+        ),
         env=dict(os.environ, LOOK_STORAGE_PATH='/tmp'),
     )
 
