@@ -5,9 +5,10 @@ import logging
 import uuid
 from wsgiref import simple_server
 
-import falcon
 import requests
 
+import falcon
+
 
 class StorageEngine:
     def get_things(self, marker, limit):
@@ -20,13 +21,12 @@ class StorageEngine:
 
 class StorageError(Exception):
     @staticmethod
-    def handle(ex, req, resp, params):
+    def handle(req, resp, ex, params):
         # TODO: Log the error, clean up, etc. before raising
         raise falcon.HTTPInternalServerError()
 
 
 class SinkAdapter:
-
     engines = {
         'ddg': 'https://duckduckgo.com',
         'y': 'https://search.yahoo.com/search',
@@ -141,7 +141,7 @@ def max_body(limit):
                 'exceed ' + str(limit) + ' bytes in length.'
             )
 
-            raise falcon.HTTPPayloadTooLarge(
+            raise falcon.HTTPContentTooLarge(
                 title='Request body is too large', description=msg
             )
 
