@@ -12,10 +12,12 @@ class ThingsResource:
         """Handles GET requests"""
         resp.status = falcon.HTTP_200  # This is the default status
         resp.content_type = falcon.MEDIA_TEXT  # Default is JSON, so override
-        resp.text = ('\nTwo things awe me most, the starry sky '
-                     'above me and the moral law within me.\n'
-                     '\n'
-                     '    ~ Immanuel Kant\n\n')
+        resp.text = (
+            '\nTwo things awe me most, the starry sky '
+            'above me and the moral law within me.\n'
+            '\n'
+            '    ~ Immanuel Kant\n\n'
+        )
 
 
 # falcon.asgi.App instances are callable ASGI apps...
