@@ -1,55 +1,48 @@
+from __future__ import annotations
+
 import asyncio
 import collections
+from enum import auto
 from enum import Enum
-from typing import Any
-from typing import Awaitable
-from typing import Callable
-from typing import Deque
-from typing import Dict
-from typing import Iterable
-from typing import Mapping
-from typing import Optional
-from typing import Union
+import re
+from typing import Any, Deque, Dict, Iterable, Mapping, Optional, Tuple, Union
 
-import falcon
 from falcon import errors
 from falcon import media
+from falcon import status_codes
+from falcon._typing import AsgiReceive
+from falcon._typing import AsgiSend
+from falcon._typing import HeaderArg
+from falcon.asgi_spec import AsgiEvent
+from falcon.asgi_spec import AsgiSendMsg
 from falcon.asgi_spec import EventType
 from falcon.asgi_spec import WSCloseCode
 from falcon.constants import WebSocketPayloadType
+from falcon.util import misc
+
+__all__ = ('WebSocket',)
 
 
-_WebSocketState = Enum('_WebSocketState', 'HANDSHAKE ACCEPTED CLOSED')
+class _WebSocketState(Enum):
+    HANDSHAKE = auto()
+    ACCEPTED = auto()
+    CLOSED = auto()
 
 
-__all__ = ['WebSocket']
+_CLIENT_DISCONNECTED_CAUSE = re.compile(r'received (\d\d\d\d)')
 
 
 class WebSocket:
-    """Represents a single WebSocket connection with a client.
-
-    Attributes:
-        ready (bool): ``True`` if the WebSocket connection has been
-            accepted and the client is still connected, ``False`` otherwise.
-        unaccepted (bool)): ``True`` if the WebSocket connection has not yet
-            been accepted, ``False`` otherwise.
-        closed (bool): ``True`` if the WebSocket connection has been closed
-            by the server or the client has disconnected.
-        subprotocols (tuple[str]): The list of subprotocol strings advertised
-            by the client, or an empty tuple if no subprotocols were
-            specified.
-        supports_accept_headers (bool): ``True`` if the ASGI server hosting
-            the app supports sending headers when accepting the WebSocket
-            connection, ``False`` otherwise.
-
-    """
+    """Represents a single WebSocket connection with a client."""
 
     __slots__ = (
         '_asgi_receive',
         '_asgi_send',
         '_buffered_receiver',
         '_close_code',
+        '_close_reasons',
         '_supports_accept_headers',
+        '_supports_reason',
         '_mh_bin_deserialize',
         '_mh_bin_serialize',
         '_mh_text_deserialize',
@@ -58,19 +51,30 @@ class WebSocket:
         'subprotocols',
     )
 
+    _asgi_receive: AsgiReceive
+    _asgi_send: AsgiSend
+    _state: _WebSocketState
+    _close_code: Optional[int]
+    subprotocols: Tuple[str, ...]
+    """The list of subprotocol strings advertised by the client, or an empty tuple if
+    no subprotocols were specified.
+    """
+
     def __init__(
         self,
         ver: str,
-        scope: dict,
-        receive: Callable[[], Awaitable[dict]],
-        send: Callable[[dict], Awaitable],
+        scope: Dict[str, Any],
+        receive: AsgiReceive,
+        send: AsgiSend,
         media_handlers: Mapping[
             WebSocketPayloadType,
-            Union[media.BinaryBaseHandlerWS, media.TextBaseHandlerWS]
+            Union[media.BinaryBaseHandlerWS, media.TextBaseHandlerWS],
         ],
         max_receive_queue: int,
+        default_close_reasons: Dict[int, str],
     ):
-        self._supports_accept_headers = (ver != '2.0')
+        self._supports_accept_headers = ver != '2.0'
+        self._supports_reason = _supports_reason(ver)
 
         # NOTE(kgriffs): Normalize the iterable to a stable tuple; note that
         #   ordering is significant, and so we preserve it here.
@@ -83,7 +87,12 @@ class WebSocket:
         #   event via one of their receive() calls, and there is no
         #   need for the added overhead.
         self._buffered_receiver = _BufferedReceiver(receive, max_receive_queue)
-        self._asgi_receive = self._buffered_receiver.receive
+        if max_receive_queue > 0:
+            self._asgi_receive = self._buffered_receiver.receive
+        else:
+            # NOTE(vytas): Pass through the receive callable bypassing the
+            #   buffered receiver in the case max_receive_queue is set to 0.
+            self._asgi_receive = receive
         self._asgi_send = send
 
         mh_text = media_handlers[WebSocketPayloadType.TEXT]
@@ -94,36 +103,49 @@ class WebSocket:
         self._mh_bin_serialize = mh_bin.serialize
         self._mh_bin_deserialize = mh_bin.deserialize
 
+        self._close_reasons = default_close_reasons
         self._state = _WebSocketState.HANDSHAKE
-        self._close_code = None  # type: Optional[int]
+        self._close_code = None
 
     @property
     def unaccepted(self) -> bool:
+        """``True`` if the WebSocket connection has not yet been accepted,
+        ``False`` otherwise.
+        """  # noqa: D205
         return self._state == _WebSocketState.HANDSHAKE
 
     @property
     def closed(self) -> bool:
+        """``True`` if the WebSocket connection has been closed by the server or the
+        client has disconnected.
+        """  # noqa: D205
         return (
-            self._state == _WebSocketState.CLOSED or
-            self._buffered_receiver.client_disconnected
+            self._state == _WebSocketState.CLOSED
+            or self._buffered_receiver.client_disconnected
         )
 
     @property
     def ready(self) -> bool:
+        """``True`` if the WebSocket connection has been accepted and the client is
+        still connected, ``False`` otherwise.
+        """  # noqa: D205
         return (
-            self._state == _WebSocketState.ACCEPTED and
-            not self._buffered_receiver.client_disconnected
+            self._state == _WebSocketState.ACCEPTED
+            and not self._buffered_receiver.client_disconnected
         )
 
     @property
     def supports_accept_headers(self) -> bool:
+        """``True`` if the ASGI server hosting the app supports sending headers when
+        accepting the WebSocket connection, ``False`` otherwise.
+        """  # noqa: D205
         return self._supports_accept_headers
 
     async def accept(
         self,
         subprotocol: Optional[str] = None,
-        headers: Optional[Union[Iterable[Iterable[str]], Mapping[str, str]]] = None
-    ):
+        headers: Optional[HeaderArg] = None,
+    ) -> None:
         """Accept the incoming WebSocket connection.
 
         If, after examining the connection's attributes (headers, advertised
@@ -144,7 +166,7 @@ class WebSocket:
                 client may choose to abandon the connection in this case,
                 if it does not receive an explicit protocol selection.
 
-            headers (Iterable[[str, str]]): An iterable of ``[name: str, value: str]``
+            headers (HeaderArg): An iterable of ``(name: str, value: str)``
                 two-item iterables, representing a collection of HTTP headers to
                 include in the handshake response. Both *name* and *value* must
                 be of type ``str`` and contain only US-ASCII characters.
@@ -189,13 +211,14 @@ class WebSocket:
                 )
 
             header_items = getattr(headers, 'items', None)
-
             if callable(header_items):
-                headers = header_items()
+                headers_iterable: Iterable[tuple[str, str]] = header_items()
+            else:
+                headers_iterable = headers  # type: ignore[assignment]
 
             event['headers'] = parsed_headers = [
                 (name.lower().encode('ascii'), value.encode('ascii'))
-                for name, value in headers  # type: ignore
+                for name, value in headers_iterable
             ]
 
             for name, __ in parsed_headers:
@@ -222,20 +245,38 @@ class WebSocket:
         #   ASGI spec committee and see if they can't come up with a better
         #   way to deal with this.
 
-    async def close(self, code: Optional[int] = None) -> None:
+    async def close(
+        self, code: Optional[int] = None, reason: Optional[str] = None
+    ) -> None:
         """Close the WebSocket connection.
 
-        This coroutine method sends a WebSocket ``CloseEvent`` to the client and
-        then proceeds to actually close the connection.
+        This coroutine method sends a WebSocket ``CloseEvent`` to the client
+        and then proceeds to actually close the connection.
 
         The responder can also use this method to deny a connection request
         simply by awaiting it instead of :meth:`~.accept`. In this case,
         the client will receive an HTTP 403 response to the handshake.
 
         Keyword Arguments:
-            code (int): The close code to use for the `CloseEvent` (default
-                1000). See also: https://developer.mozilla.org/en-US/docs/Web/API/CloseEvent
+            code (int): The close code to use for the ``CloseEvent``
+                (default 1000). See also:
+                https://developer.mozilla.org/en-US/docs/Web/API/CloseEvent/code.
+            reason(str): The string reason indicating why the server closed the
+                connection. See also:
+                https://developer.mozilla.org/en-US/docs/Web/API/CloseEvent/reason.
 
+                If there is no reason provided, Falcon will try to
+                automatically look it up from the above `code` and
+                :attr:`~WebSocketOptions.default_close_reasons`.
+
+        Note:
+            The close `reason` will only be propagated if the ASGI app server
+            supports this.
+
+            Version ``2.3``\\+ of the
+            `HTTP & WebSocket
+            <https://asgi.readthedocs.io/en/latest/specs/www.html>`__ ASGI
+            protocol is required for `reason`.
         """
 
         # NOTE(kgriffs): Do this first to be sure we clean things up
@@ -248,7 +289,7 @@ class WebSocket:
             raise ValueError('code must be an int')
         elif code < 1000:
             raise ValueError('Invalid close code. The value must be >= 1000')
-        elif (1015 <= code <= 1999 or 1004 <= code <= 1006):
+        elif 1015 <= code <= 1999 or 1004 <= code <= 1006:
             raise ValueError('Invalid close code. Only unreserved codes may be used.')
 
         # NOTE(kgriffs): Only do this after we validate the code, to avoid
@@ -256,10 +297,16 @@ class WebSocket:
         if self.closed:
             return
 
-        await self._asgi_send({
-            'type': EventType.WS_CLOSE,
-            'code': code,
-        })
+        response = {'type': EventType.WS_CLOSE, 'code': code}
+
+        reason = reason or self._close_reasons.get(code)
+        if reason and self._supports_reason:  # pragma: no py311 cover
+            # NOTE(vytas): I have verified that the below line is covered both
+            #   by multiple unit tests and E2E tests.
+            #   However, it is erroneously reported as missing on CPython 3.11.
+            response['reason'] = reason
+
+        await self._asgi_send(response)
 
         self._state = _WebSocketState.CLOSED
         self._close_code = code
@@ -267,7 +314,7 @@ class WebSocket:
     async def send_media(
         self,
         media: object,
-        payload_type: WebSocketPayloadType = WebSocketPayloadType.TEXT
+        payload_type: WebSocketPayloadType = WebSocketPayloadType.TEXT,
     ) -> None:
         """Send a serializable object to the client.
 
@@ -292,15 +339,19 @@ class WebSocket:
         self._require_accepted()
 
         if payload_type is WebSocketPayloadType.TEXT:
-            await self._send({
-                'type': EventType.WS_SEND,
-                'text': self._mh_text_serialize(media),
-            })
+            await self._send(
+                {
+                    'type': EventType.WS_SEND,
+                    'text': self._mh_text_serialize(media),
+                }
+            )
         else:
-            await self._send({
-                'type': EventType.WS_SEND,
-                'bytes': self._mh_bin_serialize(media),
-            })
+            await self._send(
+                {
+                    'type': EventType.WS_SEND,
+                    'bytes': self._mh_bin_serialize(media),
+                }
+            )
 
     async def send_text(self, payload: str) -> None:
         """Send a message to the client with a Unicode string payload.
@@ -310,17 +361,18 @@ class WebSocket:
         """
 
         self._require_accepted()
-
         # NOTE(kgriffs): We have to check ourselves because some ASGI
         #   servers are not very strict which can lead to hard-to-debug
         #   errors.
         if not isinstance(payload, str):
             raise TypeError('payload must be a string')
 
-        await self._send({
-            'type': EventType.WS_SEND,
-            'text': payload,
-        })
+        await self._send(
+            {
+                'type': EventType.WS_SEND,
+                'text': payload,
+            }
+        )
 
     async def send_data(self, payload: Union[bytes, bytearray, memoryview]) -> None:
         """Send a message to the client with a binary data payload.
@@ -329,18 +381,19 @@ class WebSocket:
             payload (Union[bytes, bytearray, memoryview]): The binary data to send.
         """
 
+        self._require_accepted()
         # NOTE(kgriffs): We have to check ourselves because some ASGI
         #   servers are not very strict which can lead to hard-to-debug
         #   errors.
         if not isinstance(payload, (bytes, bytearray, memoryview)):
             raise TypeError('payload must be a byte string')
 
-        self._require_accepted()
-
-        await self._send({
-            'type': EventType.WS_SEND,
-            'bytes': bytes(payload),
-        })
+        await self._send(
+            {
+                'type': EventType.WS_SEND,
+                'bytes': bytes(payload),
+            }
+        )
 
     async def receive_text(self) -> str:
         """Receive a message from the client with a Unicode string payload.
@@ -422,10 +475,12 @@ class WebSocket:
 
         return self._mh_bin_deserialize(data)
 
-    async def _send(self, msg: dict):
+    async def _send(self, msg: AsgiSendMsg) -> None:
         if self._buffered_receiver.client_disconnected:
             self._state = _WebSocketState.CLOSED
             self._close_code = self._buffered_receiver.client_disconnected_code
+
+        if self._state == _WebSocketState.CLOSED:
             raise errors.WebSocketDisconnected(self._close_code)
 
         try:
@@ -441,13 +496,22 @@ class WebSocket:
 
             translated_ex = self._translate_webserver_error(ex)
             if translated_ex:
-                raise translated_ex
+                # NOTE(vytas): Mark WebSocket as closed if we catch an error
+                #   upon sending. This is useful when not using the buffered
+                #   receiver, and not receiving anything at the given moment.
+                self._state = _WebSocketState.CLOSED
+                if isinstance(translated_ex, errors.WebSocketDisconnected):
+                    self._close_code = translated_ex.code
+
+                # NOTE(vytas): Use the raise from form in order to preserve
+                #   the traceback.
+                raise translated_ex from ex
 
             # NOTE(kgriffs): Re-raise other errors directly so that we don't
             #   obscure the traceback.
             raise
 
-    async def _receive(self) -> dict:
+    async def _receive(self) -> AsgiEvent:
         event = await self._asgi_receive()
 
         event_type = event['type']
@@ -464,13 +528,15 @@ class WebSocket:
 
         return event
 
-    def _require_accepted(self):
+    def _require_accepted(self) -> None:
         if self._state == _WebSocketState.HANDSHAKE:
-            raise errors.OperationNotAllowed('WebSocket connection has not yet been accepted')
+            raise errors.OperationNotAllowed(
+                'WebSocket connection has not yet been accepted'
+            )
         elif self._state == _WebSocketState.CLOSED:
             raise errors.WebSocketDisconnected(self._close_code)
 
-    def _translate_webserver_error(self, ex):
+    def _translate_webserver_error(self, ex: Exception) -> Optional[Exception]:
         s = str(ex)
 
         # NOTE(kgriffs): uvicorn or any other server using the "websockets"
@@ -481,7 +547,28 @@ class WebSocket:
         # NOTE(kgriffs): Autobahn (used by Daphne) raises a generic exception
         #   with this message
         if 'protocol accepted must be from the list' in s:
-            return ValueError('WebSocket subprotocol must be from the list sent by the client')
+            return ValueError(
+                'WebSocket subprotocol must be from the list sent by the client'
+            )
+
+        # NOTE(vytas): Per ASGI HTTP & WebSocket spec v2.4:
+        #   If send() is called on a closed connection the server should raise
+        #   a server-specific subclass of IOError.
+        # NOTE(vytas): Uvicorn 0.30.6 seems to conform to the spec only when
+        #   using the wsproto stack, it then raises an instance of
+        #   uvicorn.protocols.utils.ClientDisconnected.
+        if isinstance(ex, OSError):
+            close_code = None
+
+            # NOTE(vytas): If using the "websockets" backend, Uvicorn raises
+            #   and instance of OSError from a websockets exception like this:
+            #   "received 1001 (going away); then sent 1001 (going away)"
+            if ex.__cause__:
+                match = _CLIENT_DISCONNECTED_CAUSE.match(str(ex.__cause__))
+                if match:
+                    close_code = int(match.group(1))
+
+            return errors.WebSocketDisconnected(close_code)
 
         return None
 
@@ -490,48 +577,81 @@ class WebSocketOptions:
     """Defines a set of configurable WebSocket options.
 
     An instance of this class is exposed via :attr:`falcon.asgi.App.ws_options`
-    for configuring certain :py:class:`~.WebSocket` behaviors.
-
-    Attributes:
-        error_close_code (int): The WebSocket close code to use when an
-            unhandled error is raised while handling a WebSocket connection
-            (default ``1011``). For a list of valid close codes and ranges,
-            see also: https://tools.ietf.org/html/rfc6455#section-7.4
-        media_handlers (dict): A dict-like object for configuring media handlers
-            according to the WebSocket payload type (TEXT vs. BINARY) of a
-            given message. See also: :ref:`ws_media_handlers`.
-        max_receive_queue (int): The maximum number of incoming messages to
-            enqueue if the reception rate exceeds the consumption rate of the
-            application (default ``4``). When this limit is reached, the
-            framework will wait to accept new messages from the ASGI server
-            until the application is able to catch up.
-
-            This limit applies to Falcon's incoming message queue, and should
-            generally be kept small since the ASGI server maintains its
-            own receive queue. Falcon's queue can be disabled altogether by
-            setting `max_receive_queue` to ``0`` (see also: :ref:`ws_lost_connection`).
-
+    for configuring certain :class:`~.WebSocket` behaviors.
     """
 
-    __slots__ = ['error_close_code', 'max_receive_queue', 'media_handlers']
+    error_close_code: int
+    """The WebSocket close code to use when an unhandled error is raised while
+    handling a WebSocket connection (default ``1011``).
 
-    def __init__(self):
+    For a list of valid close codes and ranges, see also:
+    https://tools.ietf.org/html/rfc6455#section-7.4.
+    """
+    default_close_reasons: Dict[int, str]
+    """A default mapping between the Websocket close code, and the reason why
+    the connection is closed.
+    Close codes corresponding to HTTP errors are also included in this mapping.
+    """
+    media_handlers: Dict[
+        WebSocketPayloadType, Union[media.TextBaseHandlerWS, media.BinaryBaseHandlerWS]
+    ]
+    """A dict-like object for configuring media handlers according to the WebSocket
+    payload type (TEXT vs. BINARY) of a given message.
+
+    See also: :ref:`ws_media_handlers`.
+    """
+    max_receive_queue: int
+    """The maximum number of incoming messages to enqueue if the reception rate
+    exceeds the consumption rate of the application (default ``4``).
+
+    When this limit is reached, the framework will wait to accept new messages
+    from the ASGI server until the application is able to catch up.
+
+    This limit applies to Falcon's incoming message queue, and should
+    generally be kept small since the ASGI server maintains its
+    own receive queue. Falcon's queue can be disabled altogether by
+    setting `max_receive_queue` to ``0`` (see also: :ref:`ws_lost_connection`).
+    """
+
+    __slots__ = [
+        'error_close_code',
+        'default_close_reasons',
+        'max_receive_queue',
+        'media_handlers',
+    ]
+
+    _STANDARD_CLOSE_REASONS = (
+        (1000, 'Normal Closure'),
+        (1011, 'Internal Server Error'),
+        (3011, 'Internal Server Error'),
+    )
+
+    @classmethod
+    def _init_default_close_reasons(cls) -> Dict[int, str]:
+        reasons = dict(cls._STANDARD_CLOSE_REASONS)
+        for status_constant in dir(status_codes):
+            if 'HTTP_100' <= status_constant < 'HTTP_599':
+                status_line = getattr(status_codes, status_constant)
+                status_code, _, phrase = status_line.partition(' ')
+                reasons[http_status_to_ws_code(int(status_code))] = phrase
+        return reasons
+
+    def __init__(self) -> None:
         try:
             import msgpack
         except ImportError:
             msgpack = None
 
+        bin_handler: media.BinaryBaseHandlerWS
+
         if msgpack:
             bin_handler = media.MessagePackHandlerWS()
         else:
             bin_handler = media.MissingDependencyHandler(
-                'default WebSocket media handler for BINARY payloads',
-                'msgpack'
+                'default WebSocket media handler for BINARY payloads', 'msgpack'
             )
 
-        self.media_handlers: Dict[
-            WebSocketPayloadType, Union[media.TextBaseHandlerWS, media.BinaryBaseHandlerWS]
-        ] = {
+        self.media_handlers = {
             WebSocketPayloadType.TEXT: media.JSONHandlerWS(),
             WebSocketPayloadType.BINARY: bin_handler,
         }
@@ -540,7 +660,8 @@ class WebSocketOptions:
         #
         #   See also: https://developer.mozilla.org/en-US/docs/Web/API/CloseEvent
         #
-        self.error_close_code: int = WSCloseCode.SERVER_ERROR
+        self.error_close_code = WSCloseCode.SERVER_ERROR
+        self.default_close_reasons = self._init_default_close_reasons()
 
         # NOTE(kgriffs): The websockets library itself will buffer, so we keep
         #   this value fairly small by default to mitigate buffer bloat. But in
@@ -554,7 +675,7 @@ class WebSocketOptions:
         #       * https://websockets.readthedocs.io/en/stable/design.html#buffers
         #       * https://websockets.readthedocs.io/en/stable/deployment.html#buffers
         #
-        self.max_receive_queue: int = 4
+        self.max_receive_queue = 4
 
 
 class _BufferedReceiver:
@@ -576,13 +697,20 @@ class _BufferedReceiver:
         'client_disconnected_code',
     ]
 
-    def __init__(self, asgi_receive: Callable[[], Awaitable[dict]], max_queue: int):
+    _pop_message_waiter: Optional[asyncio.Future[None]]
+    _put_message_waiter: Optional[asyncio.Future[None]]
+    _pump_task: Optional[asyncio.Task[None]]
+    _messages: Deque[AsgiEvent]
+    client_disconnected: bool
+    client_disconnected_code: Optional[int]
+
+    def __init__(self, asgi_receive: AsgiReceive, max_queue: int) -> None:
         self._asgi_receive = asgi_receive
         self._max_queue = max_queue
 
-        self._loop = falcon.get_running_loop()
+        self._loop = asyncio.get_running_loop()
 
-        self._messages: Deque[dict] = collections.deque()
+        self._messages = collections.deque()
         self._pop_message_waiter = None
         self._put_message_waiter = None
 
@@ -591,12 +719,13 @@ class _BufferedReceiver:
         self.client_disconnected = False
         self.client_disconnected_code = None
 
-    def start(self):
-        if not self._pump_task:
-            self._pump_task = falcon.create_task(self._pump())
+    def start(self) -> None:
+        # NOTE(vytas): Do not start anything if buffering is disabled.
+        if self._pump_task is None and self._max_queue > 0:
+            self._pump_task = asyncio.create_task(self._pump())
 
-    async def stop(self):
-        if not self._pump_task:
+    async def stop(self) -> None:
+        if self._pump_task is None:
             return
 
         self._pump_task.cancel()
@@ -607,24 +736,25 @@ class _BufferedReceiver:
 
         self._pump_task = None
 
-    async def receive(self):
+    async def receive(self) -> AsgiEvent:
         # NOTE(kgriffs): Since this class is only used internally, we
         #   use an assertion to mitigate against framework bugs.
         #
         #   receive() may not be called again while another coroutine
         #   is already waiting for the next message.
-        assert not self._pop_message_waiter
+        assert self._pop_message_waiter is None
+        assert self._pump_task is not None
 
         # NOTE(kgriffs): Wait for a message if none are available. This pattern
         #   was borrowed from the websockets.protocol module.
         while not self._messages:
-            # -------------------------------------------------------------------------------------
+            # --------------------------------------------------------------------------
             # NOTE(kgriffs): The pattern below was borrowed from the websockets.protocol
             #   module under the BSD 3-Clause "New" or "Revised" License.
             #
-            #   Ref: https://github.com/aaugustin/websockets/blob/master/src/websockets/protocol.py
+            #   Ref: https://github.com/aaugustin/websockets/blob/master/src/websockets/protocol.py  # noqa E501
             #
-            # -------------------------------------------------------------------------------------
+            # --------------------------------------------------------------------------
 
             # PERF(kgriffs): Using a bare future like this seems to be
             #   slightly more efficient vs. something like asyncio.Event
@@ -657,20 +787,22 @@ class _BufferedReceiver:
 
         return message
 
-    async def _pump(self):
+    async def _pump(self) -> None:
         while not self.client_disconnected:
             received_event = await self._asgi_receive()
             if received_event['type'] == EventType.WS_DISCONNECT:
                 self.client_disconnected = True
-                self.client_disconnected_code = received_event.get('code', WSCloseCode.NORMAL)
+                self.client_disconnected_code = received_event.get(
+                    'code', WSCloseCode.NORMAL
+                )
 
-            # -------------------------------------------------------------------------------------
+            # --------------------------------------------------------------------------
             # NOTE(kgriffs): The pattern below was borrowed from the websockets.protocol
             #   module under the BSD 3-Clause "New" or "Revised" License.
             #
-            #   Ref: https://github.com/aaugustin/websockets/blob/master/src/websockets/protocol.py
+            #   Ref: https://github.com/aaugustin/websockets/blob/master/src/websockets/protocol.py # noqa E501
             #
-            # -------------------------------------------------------------------------------------
+            # --------------------------------------------------------------------------
             while len(self._messages) >= self._max_queue:
                 self._put_message_waiter = self._loop.create_future()
                 try:
@@ -684,3 +816,16 @@ class _BufferedReceiver:
             if self._pop_message_waiter is not None:
                 self._pop_message_waiter.set_result(None)
                 self._pop_message_waiter = None
+
+
+@misc._lru_cache_for_simple_logic(maxsize=16)
+def _supports_reason(asgi_ver: str) -> bool:
+    """Check if the websocket version support a close reason."""
+    target_ver = (2, 3)
+    current_ver = tuple(map(int, asgi_ver.split('.')))
+    return current_ver >= target_ver
+
+
+def http_status_to_ws_code(http_status: int) -> int:
+    """Convert the provided http status to a websocket close code by adding 3000."""
+    return http_status + 3000
