@@ -1,4 +1,4 @@
-# Copyright 2019-2020 by Vytautas Liuolia.
+# Copyright 2019-2024 by Vytautas Liuolia.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,28 +14,53 @@
 
 """Multipart form media handler."""
 
-import cgi
+from __future__ import annotations
+
 import re
+from typing import (
+    Any,
+    ClassVar,
+    Dict,
+    Iterator,
+    NoReturn,
+    Optional,
+    overload,
+    Tuple,
+    Type,
+    TYPE_CHECKING,
+    Union,
+)
 from urllib.parse import unquote_to_bytes
 
 from falcon import errors
+from falcon._typing import _UNSET
+from falcon._typing import UnsetOr
+from falcon.errors import MultipartParseError
 from falcon.media.base import BaseHandler
 from falcon.stream import BoundedStream
+from falcon.typing import AsyncReadableIO
+from falcon.typing import ReadableIO
 from falcon.util import BufferedReader
 from falcon.util import misc
-from falcon.util.deprecation import deprecated_args
+from falcon.util.mediatypes import parse_header
 
+if TYPE_CHECKING:
+    from falcon.asgi.multipart import MultipartForm as AsgiMultipartForm
+    from falcon.media import Handlers
+    from falcon.util.reader import BufferedReader as PyBufferedReader
 
 # TODO(vytas):
 #   * Better support for form-wide charset setting
 #   * Clean up, simplify, and optimize BufferedReader
 #   * Better documentation
 
-_ALLOWED_CONTENT_HEADERS = frozenset([
-    b'content-type',
-    b'content-disposition',
-    b'content-transfer-encoding',
-])
+_ALLOWED_CONTENT_HEADERS = frozenset(
+    [
+        b'content-type',
+        b'content-disposition',
+        b'content-transfer-encoding',
+    ]
+)
 
 _FILENAME_STAR_RFC5987 = re.compile(r"([\w-]+)'[\w]*'(.+)")
 
@@ -43,191 +68,60 @@ _CRLF = b'\r\n'
 _CRLF_CRLF = _CRLF + _CRLF
 
 
-class MultipartParseError(errors.MediaMalformedError):
-    """Represents a multipart form parsing error.
-
-    This error may refer to a malformed or truncated form, usage of deprecated
-    or unsupported features, or form parameters exceeding limits configured in
-    :class:`MultipartParseOptions`.
-
-    :class:`MultipartParseError` instances raised in this module always include
-    a short human-readable description of the error.
-
-    The cause of this exception, if any, is stored in the ``__cause__`` attribute
-    using the "raise ... from" form when raising.
-
-    Args:
-        source_error (Exception): The source exception that was the cause of this one.
-    """
-
-    # NOTE(caselit): remove the description @property in MediaMalformedError
-    description = None
-
-    @deprecated_args(allowed_positional=0)
-    def __init__(self, description=None, **kwargs):
-        errors.HTTPBadRequest.__init__(
-            self,
-            title='Malformed multipart/form-data request media',
-            description=description,
-            **kwargs
-        )
-
-
 # TODO(vytas): Consider supporting -charset- stuff.
 #   Does anyone use that (?)
 class BodyPart:
-    """Represents a body part in a multipart form.
+    """Represents a body part in a multipart form in an ASGI application.
 
     Note:
-        `BodyPart` is meant to be instantiated directly only by the
-        `MultipartForm` parser.
-
-    Attributes:
-        content_type (str): Value of the Content-Type header, or the multipart
-            form default ``text/plain`` if the header is missing.
-
-        data (bytes): Property that acts as a convenience alias for
-            :meth:`~.get_data`.
-
-
-            .. tabs::
-
-                .. tab:: WSGI
-
-                    .. code:: python
-
-                        # Equivalent to: content = part.get_data()
-                        content = part.data
-
-                .. tab:: ASGI
-
-                    The ``await`` keyword must still be added when referencing
-                    the property::
-
-                        # Equivalent to: content = await part.get_data()
-                        content = await part.data
-
-        name(str): The name parameter of the Content-Disposition header.
-            The value of the "name" parameter is the original field name from
-            the submitted HTML form.
-
-            .. note::
-               According to `RFC 7578, section 4.2
-               <https://tools.ietf.org/html/rfc7578#section-4.2>`__, each part
-               MUST include a Content-Disposition header field of type
-               "form-data", where the name parameter is mandatory.
-
-               However, Falcon will not raise any error if this parameter is
-               missing; the property value will be ``None`` in that case.
-
-        filename (str): File name if the body part is an attached file, and
-            ``None`` otherwise.
-
-        secure_filename (str): The sanitized version of `filename` using only
-            the most common ASCII characters for maximum portability and safety
-            wrt using this name as a filename on a regular file system.
-
-            If `filename` is empty or unset when referencing this property, an
-            instance of :class:`MultipartParseError` will be raised.
-
-            See also: :func:`~.secure_filename`
-
-        stream: File-like input object for reading the body part of the
-            multipart form request, if any. This object provides direct access
-            to the server's data stream and is non-seekable. The stream is
-            automatically delimited according to the multipart stream boundary.
-
-            With the exception of being buffered to keep track of the boundary,
-            the wrapped body part stream interface and behavior mimic
-            :attr:`Request.bounded_stream <falcon.Request.bounded_stream>`
-            (WSGI) and :attr:`Request.stream <falcon.asgi.Request.stream>`
-            (ASGI), respectively:
-
-            .. tabs::
-
-                .. tab:: WSGI
-
-                    Reading the whole part content:
-
-                    .. code:: python
-
-                        data = part.stream.read()
-
-                    This is also safe:
-
-                    .. code:: python
-
-                        doc = yaml.safe_load(part.stream)
-
-                .. tab:: ASGI
-
-                    Similarly to
-                    :attr:`BoundedStream <falcon.asgi.BoundedStream>`, the most
-                    efficient way to read the body part content is asynchronous
-                    iteration over part data chunks:
-
-                    .. code:: python
-
-                        async for data_chunk in part.stream:
-                            pass
-
-        media (object): Property that acts as a convenience alias for
-            :meth:`~.get_media`.
-
-            .. tabs::
-
-                .. tab:: WSGI
-
-                    .. code:: python
-
-                        # Equivalent to: deserialized_media = part.get_media()
-                        deserialized_media = req.media
-
-                .. tab:: ASGI
-
-                    The ``await`` keyword must still be added when referencing
-                    the property::
-
-                        # Equivalent to: deserialized_media = await part.get_media()
-                        deserialized_media = await part.media
-
-        text (str): Property that acts as a convenience alias for
-            :meth:`~.get_text`.
-
-            .. tabs::
-
-                .. tab:: WSGI
-
-                    .. code:: python
-
-                        # Equivalent to: decoded_text = part.get_text()
-                        decoded_text = part.text
-
-                .. tab:: ASGI
-
-                    The ``await`` keyword must still be added when referencing
-                    the property::
-
-                        # Equivalent to: decoded_text = await part.get_text()
-                        decoded_text = await part.text
+        :class:`BodyPart` is meant to be instantiated directly only by the
+        :class:`MultipartFormHandler` parser.
     """
 
-    _content_disposition = None
-    _data = None
-    _filename = None
-    _media = None
-    _name = None
+    _content_disposition: Optional[Tuple[str, Dict[str, str]]] = None
+    _data: Optional[bytes] = None
+    _filename: UnsetOr[Optional[str]] = _UNSET
+    _media: UnsetOr[Any] = _UNSET
+    _name: UnsetOr[Optional[str]] = _UNSET
 
-    def __init__(self, stream, headers, parse_options):
+    stream: PyBufferedReader
+    """File-like input object for reading the body part of the
+    multipart form request, if any. This object provides direct access
+    to the server's data stream and is non-seekable. The stream is
+    automatically delimited according to the multipart stream boundary.
+
+    With the exception of being buffered to keep track of the boundary,
+    the wrapped body part stream interface and behavior mimic
+    :attr:`Request.bounded_stream <falcon.Request.bounded_stream>`.
+
+    Reading the whole part content:
+
+    .. code:: python
+
+        data = part.stream.read()
+
+    This is also safe:
+
+    .. code:: python
+
+        doc = yaml.safe_load(part.stream)
+    """
+
+    def __init__(
+        self,
+        stream: PyBufferedReader,
+        headers: Dict[bytes, bytes],
+        parse_options: MultipartParseOptions,
+    ):
         self.stream = stream
         self._headers = headers
         self._parse_options = parse_options
 
-    def get_data(self):
+    def get_data(self) -> bytes:
         """Return the body part content bytes.
 
         The maximum number of bytes that may be read is configurable via
-        :class:`MultipartParseOptions`, and a :class:`.MultipartParseError` is
+        :class:`.MultipartParseOptions`, and a :class:`.MultipartParseError` is
         raised if the body part is larger that this size.
 
         The size limit guards against reading unexpectedly large amount of data
@@ -252,7 +146,7 @@ class BodyPart:
 
         return self._data
 
-    def get_text(self):
+    def get_text(self) -> Optional[str]:
         """Return the body part content decoded as a text string.
 
         Text is decoded from the part content (as returned by
@@ -266,7 +160,7 @@ class BodyPart:
 
         If decoding fails due to invalid `data` bytes (for the specified
         encoding), or the specified encoding itself is unsupported, a
-        :class:`MultipartParseError` will be raised when referencing this
+        :class:`.MultipartParseError` will be raised when referencing this
         property.
 
         Note:
@@ -277,7 +171,7 @@ class BodyPart:
             str: The part decoded as a text string provided the part is
             encoded as ``text/plain``, ``None`` otherwise.
         """
-        content_type, options = cgi.parse_header(self.content_type)
+        content_type, options = parse_header(self.content_type)
         if content_type != 'text/plain':
             return None
 
@@ -290,7 +184,11 @@ class BodyPart:
             ) from err
 
     @property
-    def content_type(self):
+    def content_type(self) -> str:
+        """Value of the Content-Type header.
+
+        When the header is missing returns the multipart form default ``text/plain``.
+        """
         # NOTE(vytas): RFC 7578, section 4.4.
         #   Each part MAY have an (optional) "Content-Type" header field, which
         #   defaults to "text/plain".
@@ -298,12 +196,12 @@ class BodyPart:
         return value.decode('ascii')
 
     @property
-    def filename(self):
-        if self._filename is None:
-
+    def filename(self) -> Optional[str]:
+        """File name if the body part is an attached file, and ``None`` otherwise."""
+        if self._filename is _UNSET:
             if self._content_disposition is None:
                 value = self._headers.get(b'content-disposition', b'')
-                self._content_disposition = cgi.parse_header(value.decode())
+                self._content_disposition = parse_header(value.decode())
 
             _, params = self._content_disposition
 
@@ -311,97 +209,151 @@ class BodyPart:
             #   been spotted in the wild, even though RFC 7578 forbids it.
             match = _FILENAME_STAR_RFC5987.match(params.get('filename*', ''))
             if match:
-                charset, value = match.groups()
+                charset, filename_raw = match.groups()
                 try:
-                    self._filename = unquote_to_bytes(value).decode(charset)
+                    self._filename = unquote_to_bytes(filename_raw).decode(charset)
                 except (ValueError, LookupError) as err:
                     raise MultipartParseError(
                         description='invalid text or charset: {}'.format(charset)
                     ) from err
             else:
-                value = params.get('filename')
-                if value is None:
-                    return None
-                self._filename = value
+                self._filename = params.get('filename')
 
         return self._filename
 
     @property
-    def secure_filename(self):
+    def secure_filename(self) -> str:
+        """The sanitized version of `filename` using only the most common ASCII
+        characters for maximum portability and safety wrt using this name as a
+        filename on a regular file system.
+
+        If `filename` is empty or unset when referencing this property, an
+        instance of :class:`.MultipartParseError` will be raised.
+
+        See also: :func:`~.secure_filename`
+        """  # noqa: D205
         try:
-            return misc.secure_filename(self.filename)
+            return misc.secure_filename(self.filename or '')
         except ValueError as ex:
             raise MultipartParseError(description=str(ex)) from ex
 
     @property
-    def name(self):
-        if self._name is None:
+    def name(self) -> Optional[str]:
+        """The name parameter of the Content-Disposition header.
 
+        The value of the "name" parameter is the original field name from
+        the submitted HTML form.
+
+        .. note::
+            According to `RFC 7578, section 4.2
+            <https://tools.ietf.org/html/rfc7578#section-4.2>`__, each part
+            MUST include a Content-Disposition header field of type
+            "form-data", where the name parameter is mandatory.
+
+            However, Falcon will not raise any error if this parameter is
+            missing; the property value will be ``None`` in that case.
+        """
+        if self._name is _UNSET:
             if self._content_disposition is None:
                 value = self._headers.get(b'content-disposition', b'')
-                self._content_disposition = cgi.parse_header(value.decode())
+                self._content_disposition = parse_header(value.decode())
 
             _, params = self._content_disposition
             self._name = params.get('name')
 
         return self._name
 
-    def get_media(self):
+    def get_media(self) -> Any:
         """Return a deserialized form of the multipart body part.
 
         When called, this method will attempt to deserialize the body part
         stream using the Content-Type header as well as the media-type handlers
         configured via :class:`MultipartParseOptions`.
 
-        .. tabs::
+        The result will be cached and returned in subsequent calls::
 
-            .. tab:: WSGI
-
-                The result will be cached and returned in subsequent calls::
-
-                    deserialized_media = part.get_media()
-
-            .. tab:: ASGI
-
-                The result will be cached and returned in subsequent calls::
-
-                    deserialized_media = await part.get_media()
+            deserialized_media = part.get_media()
 
         Returns:
             object: The deserialized media representation.
         """
-        if self._media is None:
+        if self._media is _UNSET:
             handler, _, _ = self._parse_options.media_handlers._resolve(
-                self.content_type, 'text/plain')
+                self.content_type, 'text/plain'
+            )
 
             try:
-                self._media = handler.deserialize(
-                    self.stream, self.content_type, None)
+                self._media = handler.deserialize(self.stream, self.content_type, None)
             finally:
                 if handler.exhaust_stream:
                     self.stream.exhaust()
 
         return self._media
 
-    data = property(get_data)
-    media = property(get_media)
-    text = property(get_text)
+    data: bytes = property(get_data)  # type: ignore[assignment]
+    """Property that acts as a convenience alias for :meth:`~.get_data`.
+
+    .. code:: python
+
+        # Equivalent to: content = part.get_data()
+        content = part.data
+    """
+    media: Any = property(get_media)
+    """Property that acts as a convenience alias for :meth:`~.get_media`.
+
+    .. code:: python
+
+        # Equivalent to: deserialized_media = part.get_media()
+        deserialized_media = req.media
+    """
+    text: str = property(get_text)  # type: ignore[assignment]
+    """Property that acts as a convenience alias for :meth:`~.get_text`.
+
+    .. code:: python
+
+        # Equivalent to: decoded_text = part.get_text()
+        decoded_text = part.text
+    """
 
 
 class MultipartForm:
+    """Iterable object that returns each form part as :class:`BodyPart` instances.
 
-    def __init__(self, stream, boundary, content_length, parse_options):
+    Typical usage illustrated below::
+
+        def on_post(self, req: Request, resp: Response) -> None:
+            form: MultipartForm = req.get_media()
+
+            for part in form:
+                if part.name == 'foo':
+                    ...
+                else:
+                    ...
+
+    Note:
+        :class:`MultipartForm` is meant to be instantiated directly only by the
+        :class:`MultipartFormHandler` parser.
+    """
+
+    def __init__(
+        self,
+        stream: ReadableIO,
+        boundary: bytes,
+        content_length: Optional[int],
+        parse_options: MultipartParseOptions,
+    ) -> None:
         # NOTE(vytas): More lenient check whether the provided stream is not
         #   already an instance of BufferedReader.
         # This approach makes testing both the Cythonized and pure-Python
         #   streams easier within the same test/benchmark suite.
         if not hasattr(stream, 'read_until'):
+            assert content_length is not None
             if isinstance(stream, BoundedStream):
                 stream = BufferedReader(stream.stream.read, content_length)
             else:
                 stream = BufferedReader(stream.read, content_length)
 
-        self._stream = stream
+        self._stream: PyBufferedReader = stream  # type: ignore[assignment]
         self._boundary = boundary
         # NOTE(vytas): Here self._dash_boundary is not prepended with CRLF
         #   (yet) for parsing the prologue. The CRLF will be prepended later to
@@ -410,7 +362,7 @@ class MultipartForm:
         self._dash_boundary = b'--' + boundary
         self._parse_options = parse_options
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[BodyPart]:
         prologue = True
         delimiter = self._dash_boundary
         stream = self._stream
@@ -431,23 +383,35 @@ class MultipartForm:
                     delimiter = _CRLF + delimiter
                     prologue = False
 
-                separator = stream.read_until(_CRLF, 2, consume_delimiter=True)
-                if separator == b'--':
-                    # NOTE(vytas): boundary delimiter + '--\r\n' signals the
-                    # end of a multipart form.
+                # NOTE(vytas): Interpretations of RFC 2046, Appendix A, vary
+                #   as to whether the closing `--` must be followed by CRLF.
+                #   While the absolute majority of HTTP clients and browsers
+                #   do append it as a common convention, it seems that this is
+                #   not mandated by the RFC, so we do not require it either.
+                # NOTE(vytas): Certain versions of the Undici client
+                #   (Node's fetch implementation) do not follow the convention.
+                if stream.peek(2) == b'--':
+                    # NOTE(vytas): boundary delimiter + '--' signals the end of
+                    #   a multipart form.
+                    stream.read(2)
                     break
-                elif separator:
-                    raise MultipartParseError(description='unexpected form structure')
+
+                stream.read_until(_CRLF, 0, consume_delimiter=True)
 
             except errors.DelimiterError as err:
-                raise MultipartParseError(description='unexpected form structure') from err
+                raise MultipartParseError(
+                    description='unexpected form structure'
+                ) from err
 
-            headers = {}
+            headers: Dict[bytes, bytes] = {}
             try:
                 headers_block = stream.read_until(
-                    _CRLF_CRLF, max_headers_size, consume_delimiter=True)
+                    _CRLF_CRLF, max_headers_size, consume_delimiter=True
+                )
             except errors.DelimiterError as err:
-                raise MultipartParseError(description='incomplete body part headers') from err
+                raise MultipartParseError(
+                    description='incomplete body part headers'
+                ) from err
 
             for line in headers_block.split(_CRLF):
                 name, sep, value = line.partition(b': ')
@@ -464,8 +428,10 @@ class MultipartForm:
                     #   bodies have been discovered.
                     if name == b'content-transfer-encoding' and value != b'binary':
                         raise MultipartParseError(
-                            description=('the deprecated Content-Transfer-Encoding '
-                                         'header field is unsupported')
+                            description=(
+                                'the deprecated Content-Transfer-Encoding '
+                                'header field is unsupported'
+                            )
                         )
                     # NOTE(vytas): RFC 7578, section 4.8.
                     #   Other header fields MUST NOT be included and MUST be
@@ -479,8 +445,7 @@ class MultipartForm:
                     description='maximum number of form body parts exceeded'
                 )
 
-            yield BodyPart(stream.delimit(delimiter), headers,
-                           self._parse_options)
+            yield BodyPart(stream.delimit(delimiter), headers, self._parse_options)
 
 
 class MultipartFormHandler(BaseHandler):
@@ -501,23 +466,52 @@ class MultipartFormHandler(BaseHandler):
     For examples on parsing the request form, see also: :ref:`multipart`.
     """
 
-    _ASGI_MULTIPART_FORM = None
+    _ASGI_MULTIPART_FORM: ClassVar[Type[AsgiMultipartForm]]
 
-    def __init__(self, parse_options=None):
+    parse_options: MultipartParseOptions
+    """Configuration options for the multipart form parser and instances of
+    :class:`~falcon.media.multipart.BodyPart` it yields.
+
+    See also: :ref:`multipart_parser_conf`.
+    """
+
+    def __init__(self, parse_options: Optional[MultipartParseOptions] = None) -> None:
         self.parse_options = parse_options or MultipartParseOptions()
 
-    def _deserialize_form(self, stream, content_type, content_length,
-                          form_cls=MultipartForm):
-        if not form_cls:
-            raise NotImplementedError
+    @overload
+    def _deserialize_form(
+        self,
+        stream: ReadableIO,
+        content_type: Optional[str],
+        content_length: Optional[int],
+        form_cls: Type[MultipartForm] = ...,
+    ) -> MultipartForm: ...
 
-        _, options = cgi.parse_header(content_type)
+    @overload
+    def _deserialize_form(
+        self,
+        stream: AsyncReadableIO,
+        content_type: Optional[str],
+        content_length: Optional[int],
+        form_cls: Type[AsgiMultipartForm] = ...,
+    ) -> AsgiMultipartForm: ...
+
+    def _deserialize_form(
+        self,
+        stream: Union[ReadableIO, AsyncReadableIO],
+        content_type: Optional[str],
+        content_length: Optional[int],
+        form_cls: Type[Union[MultipartForm, AsgiMultipartForm]] = MultipartForm,
+    ) -> Union[MultipartForm, AsgiMultipartForm]:
+        assert content_type is not None
+        _, options = parse_header(content_type)
         try:
             boundary = options['boundary']
         except KeyError:
             raise errors.HTTPInvalidHeader(
                 'No boundary specifier found in {!r}'.format(content_type),
-                'Content-Type')
+                'Content-Type',
+            )
 
         # NOTE(vytas): RFC 2046, section 5.1.
         #   If a boundary delimiter line appears to end with white space, the
@@ -532,19 +526,30 @@ class MultipartFormHandler(BaseHandler):
         if not 1 <= len(boundary) <= 70:
             raise errors.HTTPInvalidHeader(
                 'The boundary parameter must consist of 1 to 70 characters',
-                'Content-Type')
+                'Content-Type',
+            )
 
-        return form_cls(stream, boundary.encode(), content_length,
-                        self.parse_options)
+        return form_cls(stream, boundary.encode(), content_length, self.parse_options)  # type: ignore[arg-type]
 
-    def deserialize(self, stream, content_type, content_length):
+    def deserialize(
+        self,
+        stream: ReadableIO,
+        content_type: Optional[str],
+        content_length: Optional[int],
+    ) -> MultipartForm:
         return self._deserialize_form(stream, content_type, content_length)
 
-    async def deserialize_async(self, stream, content_type, content_length):
-        return self._deserialize_form(stream, content_type, content_length,
-                                      form_cls=self._ASGI_MULTIPART_FORM)
+    async def deserialize_async(
+        self,
+        stream: AsyncReadableIO,
+        content_type: Optional[str],
+        content_length: Optional[int],
+    ) -> AsgiMultipartForm:
+        return self._deserialize_form(
+            stream, content_type, content_length, form_cls=self._ASGI_MULTIPART_FORM
+        )
 
-    def serialize(self, media, content_type):
+    def serialize(self, media: object, content_type: str) -> NoReturn:
         raise NotImplementedError('multipart form serialization unsupported')
 
 
@@ -553,32 +558,51 @@ class MultipartFormHandler(BaseHandler):
 class MultipartParseOptions:
     """Defines a set of configurable multipart form parser options.
 
-    Attributes:
-        default_charset (str): The default character encoding for
-            :meth:`text fields <BodyPart.get_text>` (default: ``utf-8``).
+    An instance of this class is exposed via the
+    :attr:`MultipartFormHandler.parse_options
+    <falcon.media.MultipartFormHandler.parse_options>` attribute.
+    The handler's options are also passed down to every :class:`BodyPart`
+    it instantiates.
 
-        max_body_part_count (int): The maximum number of body parts in the form
-            (default: 64). If the form contains more parts than this number,
-            an instance of :class:`MultipartParseError` will be raised. If this
-            option is set to 0, no limit will be imposed by the parser.
-
-        max_body_part_buffer_size (int): The maximum number of bytes to buffer
-            and return when the :meth:`BodyPart.get_data` method is called
-            (default: 1 MiB). If the body part size exceeds this value, an
-            instance of :class:`MultipartParseError` will be raised.
-
-        max_body_part_headers_size (int): The maximum size (in bytes) of the
-            body part headers structure (default: 8192). If the body part
-            headers size exceeds this value, an instance of
-            :class:`MultipartParseError` will be raised.
-
-        media_handlers (Handlers): A dict-like object for configuring the
-            media-types to handle. By default, handlers are provided for the
-            ``application/json`` and ``application/x-www-form-urlencoded``
-            media types.
+    See also: :ref:`multipart_parser_conf`.
     """
 
-    _DEFAULT_HANDLERS = None
+    default_charset: str
+    """The default character encoding for
+    :meth:`text fields <BodyPart.get_text>` (default ``utf-8``).
+    """
+    max_body_part_count: int
+    """The maximum number of body parts in the form (default ``64``).
+
+    If the form contains more parts than this number, an instance of
+    :class:`.MultipartParseError` will be raised. If this option is set to 0,
+    no limit will be imposed by the parser.
+    """
+    max_body_part_buffer_size: int
+    """The maximum number of bytes to buffer and return when the
+    :meth:`BodyPart.get_data` method is called (default ``1 MiB``).
+
+    If the body part size exceeds this value, an instance of
+    :class:`.MultipartParseError` will be raised.
+    """
+    max_body_part_headers_size: int
+    """The maximum size (in bytes) of the body part headers structure
+    (default ``8192``).
+
+    If the body part headers size exceeds this value, an instance of
+    :class:`.MultipartParseError` will be raised.
+    """
+    media_handlers: Handlers
+    """A dict-like object for configuring the media-types to handle.
+
+    By default, handlers are provided for the ``application/json`` and
+    ``application/x-www-form-urlencoded`` media types.
+    """
+
+    if TYPE_CHECKING:
+        _DEFAULT_HANDLERS: ClassVar[Handlers]
+    else:
+        _DEFAULT_HANDLERS = None
 
     __slots__ = (
         'default_charset',
@@ -588,9 +612,12 @@ class MultipartParseOptions:
         'media_handlers',
     )
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.default_charset = 'utf-8'
         self.max_body_part_buffer_size = 1024 * 1024
         self.max_body_part_count = 64
         self.max_body_part_headers_size = 8192
-        self.media_handlers = self._DEFAULT_HANDLERS
+        # NOTE(myusko,vytas): Here we create a copy of _DEFAULT_HANDLERS in
+        #   order to prevent the modification of the class variable whenever
+        #   parse_options.media_handlers are customized.
+        self.media_handlers = self._DEFAULT_HANDLERS.copy()
