@@ -1,6 +1,6 @@
-from typing import Iterable
-from typing import Optional
-from typing import Union
+from __future__ import annotations
+
+from typing import Any, Iterable, Optional, Union
 
 from .request import Request
 from .response import Response
@@ -17,6 +17,15 @@ class CORSMiddleware(object):
     * https://developer.mozilla.org/en-US/docs/Web/HTTP/CORS
     * https://www.w3.org/TR/cors/#resource-processing-model
 
+    Note:
+        Falcon will automatically add OPTIONS responders if they are missing from the
+        responder instances added to the routes. When providing a custom ``on_options``
+        method, the ``Allow`` headers in the response should be set to the allowed
+        method values. If the ``Allow`` header is missing from the response,
+        this middleware will deny the preflight request.
+
+        This is also valid when using a sink function.
+
     Keyword Arguments:
         allow_origins (Union[str, Iterable[str]]): List of origins to allow (case
             sensitive). The string ``'*'`` acts as a wildcard, matching every origin.
@@ -75,7 +84,9 @@ class CORSMiddleware(object):
                 )
         self.allow_credentials = allow_credentials
 
-    def process_response(self, req: Request, resp: Response, resource, req_succeeded):
+    def process_response(
+        self, req: Request, resp: Response, resource: object, req_succeeded: bool
+    ) -> None:
         """Implement the CORS policy for all routes.
 
         This middleware provides a simple out-of-the box CORS policy,
@@ -108,7 +119,6 @@ class CORSMiddleware(object):
             and req.method == 'OPTIONS'
             and req.get_header('Access-Control-Request-Method')
         ):
-
             # NOTE(kgriffs): This is a CORS preflight request. Patch the
             #   response accordingly.
 
@@ -119,9 +129,17 @@ class CORSMiddleware(object):
                 'Access-Control-Request-Headers', default='*'
             )
 
-            resp.set_header('Access-Control-Allow-Methods', allow)
-            resp.set_header('Access-Control-Allow-Headers', allow_headers)
-            resp.set_header('Access-Control-Max-Age', '86400')  # 24 hours
+            if allow is None:
+                # there is no allow set, remove all access control headers
+                resp.delete_header('Access-Control-Allow-Methods')
+                resp.delete_header('Access-Control-Allow-Headers')
+                resp.delete_header('Access-Control-Max-Age')
+                resp.delete_header('Access-Control-Expose-Headers')
+                resp.delete_header('Access-Control-Allow-Origin')
+            else:
+                resp.set_header('Access-Control-Allow-Methods', allow)
+                resp.set_header('Access-Control-Allow-Headers', allow_headers)
+                resp.set_header('Access-Control-Max-Age', '86400')  # 24 hours
 
-    async def process_response_async(self, *args):
+    async def process_response_async(self, *args: Any) -> None:
         self.process_response(*args)
