@@ -23,11 +23,12 @@ from falcon.routing.compiled import CompiledRouter
 from falcon.routing.compiled import CompiledRouterOptions
 from falcon.routing.converters import BaseConverter
 from falcon.routing.converters import DateTimeConverter
+from falcon.routing.converters import FloatConverter
 from falcon.routing.converters import IntConverter
+from falcon.routing.converters import PathConverter
 from falcon.routing.converters import UUIDConverter
 from falcon.routing.static import StaticRoute
 from falcon.routing.static import StaticRouteAsync
-from falcon.routing.util import compile_uri_template
 from falcon.routing.util import map_http_methods
 from falcon.routing.util import set_default_responders
 
