@@ -14,12 +14,28 @@
 
 """Default routing engine."""
 
+from __future__ import annotations
+
 from collections import UserDict
 from inspect import iscoroutinefunction
 import keyword
 import re
 from threading import Lock
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Pattern,
+    Set,
+    Tuple,
+    Type,
+    TYPE_CHECKING,
+    Union,
+)
 
+from falcon._typing import MethodDict
 from falcon.routing import converters
 from falcon.routing.util import map_http_methods
 from falcon.routing.util import set_default_responders
@@ -27,8 +43,10 @@ from falcon.util.misc import is_python_func
 from falcon.util.sync import _should_wrap_non_coroutines
 from falcon.util.sync import wrap_sync_to_async
 
-if False:  # TODO: switch to TYPE_CHECKING once support for py3.5 is dropped
-    from typing import Any
+if TYPE_CHECKING:
+    from falcon import Request
+
+    _CxElement = Union['_CxParent', '_CxChild']
 
 _TAB_STR = ' ' * 4
 _FIELD_PATTERN = re.compile(
@@ -85,10 +103,10 @@ class CompiledRouter:
         '_compile_lock',
     )
 
-    def __init__(self):
-        self._ast = None
-        self._converters = None
-        self._finder_src = None
+    def __init__(self) -> None:
+        self._ast: _CxParent = _CxParent()
+        self._converters: List[converters.BaseConverter] = []
+        self._finder_src: str = ''
 
         self._options = CompiledRouterOptions()
 
@@ -96,9 +114,9 @@ class CompiledRouter:
         # here to reduce lookup time.
         self._converter_map = self._options.converters.data
 
-        self._patterns = None
-        self._return_values = None
-        self._roots = []
+        self._patterns: List[Pattern] = []
+        self._return_values: List[CompiledRouterNode] = []
+        self._roots: List[CompiledRouterNode] = []
 
         # NOTE(caselit): set _find to the delayed compile method to ensure that
         # compile is called when the router is first used
@@ -106,18 +124,18 @@ class CompiledRouter:
         self._compile_lock = Lock()
 
     @property
-    def options(self):
+    def options(self) -> CompiledRouterOptions:
         return self._options
 
     @property
-    def finder_src(self):
+    def finder_src(self) -> str:
         # NOTE(caselit): ensure that the router is actually compiled before
         # returning the finder source, since the current value may be out of
         # date
         self.find('/')
         return self._finder_src
 
-    def map_http_methods(self, resource, **kwargs):
+    def map_http_methods(self, resource: object, **kwargs: Any) -> MethodDict:
         """Map HTTP methods (e.g., GET, POST) to methods of a resource object.
 
         This method is called from :meth:`~.add_route` and may be overridden to
@@ -146,7 +164,9 @@ class CompiledRouter:
 
         return map_http_methods(resource, suffix=kwargs.get('suffix', None))
 
-    def add_route(self, uri_template, resource, **kwargs):
+    def add_route(  # noqa: C901
+        self, uri_template: str, resource: object, **kwargs: Any
+    ) -> None:
         """Add a route between a URI path template and a resource.
 
         This method may be overridden to customize how a route is added.
@@ -185,7 +205,7 @@ class CompiledRouter:
 
         # NOTE(kgriffs): falcon.asgi.App injects this private kwarg; it is
         #   only intended to be used internally.
-        asgi = kwargs.get('_asgi', False)
+        asgi: bool = kwargs.get('_asgi', False)
 
         method_map = self.map_http_methods(resource, **kwargs)
 
@@ -199,15 +219,28 @@ class CompiledRouter:
         # NOTE(kgriffs): Fields may have whitespace in them, so sub
         # those before checking the rest of the URI template.
         if re.search(r'\s', _FIELD_PATTERN.sub('{FIELD}', uri_template)):
-            raise ValueError('URI templates may not include whitespace.')
+            raise UnacceptableRouteError('URI templates may not include whitespace.')
 
         path = uri_template.lstrip('/').split('/')
 
-        used_names = set()
+        used_names: Set[str] = set()
         for segment in path:
             self._validate_template_segment(segment, used_names)
 
-        def insert(nodes, path_index=0):
+        def find_cmp_converter(node: CompiledRouterNode) -> Optional[Tuple[str, str]]:
+            value = [
+                (field, converter)
+                for field, converter, _ in node.var_converter_map
+                if converters._consumes_multiple_segments(
+                    self._converter_map[converter]
+                )
+            ]
+            if value:
+                return value[0]
+            else:
+                return None
+
+        def insert(nodes: List[CompiledRouterNode], path_index: int = 0) -> None:
             for node in nodes:
                 segment = path[path_index]
                 if node.matches(segment):
@@ -218,12 +251,17 @@ class CompiledRouter:
                         node.resource = resource
                         node.uri_template = uri_template
                     else:
+                        cpc = find_cmp_converter(node)
+                        if cpc:
+                            raise UnacceptableRouteError(
+                                _NO_CHILDREN_ERR.format(uri_template, *cpc)
+                            )
                         insert(node.children, path_index)
 
                     return
 
                 if node.conflicts_with(segment):
-                    raise ValueError(
+                    raise UnacceptableRouteError(
                         'The URI template for this route is inconsistent or conflicts '
                         "with another route's template. This is usually caused by "
                         'configuring a field converter differently for the same field '
@@ -236,12 +274,27 @@ class CompiledRouter:
             # exist and needs to be created. This builds a new branch of the
             # routing tree recursively until it reaches the new node leaf.
             new_node = CompiledRouterNode(path[path_index])
+            if new_node.is_complex:
+                cpc = find_cmp_converter(new_node)
+                if cpc:
+                    raise UnacceptableRouteError(
+                        'Cannot use converter "{1}" of variable "{0}" in a template '
+                        'that includes other characters or variables.'.format(*cpc)
+                    )
             nodes.append(new_node)
             if path_index == len(path) - 1:
                 new_node.method_map = method_map
                 new_node.resource = resource
                 new_node.uri_template = uri_template
             else:
+                cpc = find_cmp_converter(new_node)
+                if cpc:
+                    # NOTE(caselit): assume success and remove the node if it's not
+                    # supported to avoid leaving the router in a broken state.
+                    nodes.remove(new_node)
+                    raise UnacceptableRouteError(
+                        _NO_CHILDREN_ERR.format(uri_template, *cpc)
+                    )
                 insert(new_node.children, path_index + 1)
 
         insert(self._roots)
@@ -252,7 +305,11 @@ class CompiledRouter:
         else:
             self._find = self._compile_and_find
 
-    def find(self, uri, req=None):
+    # NOTE(caselit): keep Request as string otherwise sphinx complains that it resolves
+    # to multiple classes, since the symbol is imported only for type check.
+    def find(
+        self, uri: str, req: Optional['Request'] = None
+    ) -> Optional[Tuple[object, MethodDict, Dict[str, Any], Optional[str]]]:
         """Search for a route that matches the given partial URI.
 
         Args:
@@ -271,13 +328,13 @@ class CompiledRouter:
         """
 
         path = uri.lstrip('/').split('/')
-        params = {}
-        node = self._find(
+        params: Dict[str, Any] = {}
+        node: Optional[CompiledRouterNode] = self._find(
             path, self._return_values, self._patterns, self._converters, params
         )
 
         if node is not None:
-            return node.resource, node.method_map, params, node.uri_template
+            return node.resource, node.method_map or {}, params, node.uri_template
         else:
             return None
 
@@ -285,21 +342,16 @@ class CompiledRouter:
     # Private
     # -----------------------------------------------------------------
 
-    def _require_coroutine_responders(self, method_map):
+    def _require_coroutine_responders(self, method_map: MethodDict) -> None:
         for method, responder in method_map.items():
             # NOTE(kgriffs): We don't simply wrap non-async functions
-            #   since they likely peform relatively long blocking
+            #   since they likely perform relatively long blocking
             #   operations that need to be explicitly made non-blocking
             #   by the developer; raising an error helps highlight this
             #   issue.
             if not iscoroutinefunction(responder) and is_python_func(responder):
                 if _should_wrap_non_coroutines():
-
-                    def let(responder=responder):
-                        method_map[method] = wrap_sync_to_async(responder)
-
-                    let()
-
+                    method_map[method] = wrap_sync_to_async(responder)
                 else:
                     msg = (
                         'The {} responder must be a non-blocking '
@@ -309,10 +361,10 @@ class CompiledRouter:
                     msg = msg.format(responder)
                     raise TypeError(msg)
 
-    def _require_non_coroutine_responders(self, method_map):
+    def _require_non_coroutine_responders(self, method_map: MethodDict) -> None:
         for method, responder in method_map.items():
             # NOTE(kgriffs): We don't simply wrap non-async functions
-            #   since they likely peform relatively long blocking
+            #   since they likely perform relatively long blocking
             #   operations that need to be explicitly made non-blocking
             #   by the developer; raising an error helps highlight this
             #   issue.
@@ -325,7 +377,7 @@ class CompiledRouter:
                 msg = msg.format(responder)
                 raise TypeError(msg)
 
-    def _validate_template_segment(self, segment, used_names):
+    def _validate_template_segment(self, segment: str, used_names: Set[str]) -> None:
         """Validate a single path segment of a URI template.
 
         1. Ensure field names are valid Python identifiers, since they
@@ -349,7 +401,7 @@ class CompiledRouter:
                     'Field names must be valid identifiers ("{0}" is not valid)'
                 )
                 msg = msg_template.format(name)
-                raise ValueError(msg)
+                raise UnacceptableRouteError(msg)
 
             if name in used_names:
                 msg_template = (
@@ -357,37 +409,37 @@ class CompiledRouter:
                     '("{0}" was used more than once)'
                 )
                 msg = msg_template.format(name)
-                raise ValueError(msg)
+                raise UnacceptableRouteError(msg)
 
             used_names.add(name)
 
             if field.group('cname_sep') == ':':
                 msg = 'Missing converter for field "{0}"'.format(name)
-                raise ValueError(msg)
+                raise UnacceptableRouteError(msg)
 
             name = field.group('cname')
             if name:
                 if name not in self._converter_map:
                     msg = 'Unknown converter: "{0}"'.format(name)
-                    raise ValueError(msg)
+                    raise UnacceptableRouteError(msg)
                 try:
                     self._instantiate_converter(
                         self._converter_map[name], field.group('argstr')
                     )
                 except Exception as e:
                     msg = 'Cannot instantiate converter "{}"'.format(name)
-                    raise ValueError(msg) from e
+                    raise UnacceptableRouteError(msg) from e
 
-    def _generate_ast(
+    def _generate_ast(  # noqa: C901
         self,
-        nodes: list,
-        parent,
-        return_values: list,
-        patterns: list,
-        params_stack: list,
-        level=0,
-        fast_return=True,
-    ):
+        nodes: List[CompiledRouterNode],
+        parent: _CxParent,
+        return_values: List[CompiledRouterNode],
+        patterns: List[Pattern],
+        params_stack: List[_CxElement],
+        level: int = 0,
+        fast_return: bool = True,
+    ) -> None:
         """Generate a coarse AST for the router."""
         # NOTE(caselit): setting of the parameters in the params dict is delayed until
         # a match has been found by adding them to the param_stack. This way superfluous
@@ -423,25 +475,25 @@ class CompiledRouter:
 
                 fast_return = not found_var_nodes
 
-        construct = None  # type: Any
-        setter = None  # type: Any
         original_params_stack = params_stack.copy()
         for node in nodes:
             params_stack = original_params_stack.copy()
+            consume_multiple_segments = False
             if node.is_var:
                 if node.is_complex:
                     # NOTE(richardolsson): Complex nodes are nodes which
                     # contain anything more than a single literal or variable,
                     # and they need to be checked using a pre-compiled regular
                     # expression.
+                    assert node.var_pattern
                     pattern_idx = len(patterns)
                     patterns.append(node.var_pattern)
 
-                    construct = _CxIfPathSegmentPattern(
+                    cx_segment = _CxIfPathSegmentPattern(
                         level, pattern_idx, node.var_pattern.pattern
                     )
-                    parent.append_child(construct)
-                    parent = construct
+                    parent.append_child(cx_segment)
+                    parent = cx_segment
 
                     if node.var_converter_map:
                         parent.append_child(_CxPrefetchGroupsFromPatternMatch())
@@ -450,21 +502,21 @@ class CompiledRouter:
                         )
 
                     else:
-                        construct = _CxVariableFromPatternMatch(len(params_stack) + 1)
-                        setter = _CxSetParamsFromDict(construct.dict_variable_name)
-                        params_stack.append(setter)
-                        parent.append_child(construct)
+                        cx_pattern = _CxVariableFromPatternMatch(len(params_stack) + 1)
+                        params_stack.append(
+                            _CxSetParamsFromDict(cx_pattern.dict_variable_name)
+                        )
+                        parent.append_child(cx_pattern)
 
                 else:
                     # NOTE(kgriffs): Simple nodes just capture the entire path
-                    # segment as the value for the param.
+                    # segment as the value for the param. They have a var_name defined
 
+                    field_name = node.var_name
+                    assert field_name is not None
                     if node.var_converter_map:
                         assert len(node.var_converter_map) == 1
 
-                        parent.append_child(_CxSetFragmentFromPath(level))
-
-                        field_name = node.var_name
                         __, converter_name, converter_argstr = node.var_converter_map[0]
                         converter_class = self._converter_map[converter_name]
 
@@ -473,19 +525,25 @@ class CompiledRouter:
                         )
                         converter_idx = len(self._converters)
                         self._converters.append(converter_obj)
+                        if converters._consumes_multiple_segments(converter_obj):
+                            consume_multiple_segments = True
+                            parent.append_child(_CxSetFragmentFromRemainingPaths(level))
+                        else:
+                            parent.append_child(_CxSetFragmentFromPath(level))
 
-                        construct = _CxIfConverterField(
+                        cx_converter = _CxIfConverterField(
                             len(params_stack) + 1, converter_idx
                         )
-                        setter = _CxSetParamFromValue(
-                            field_name, construct.field_variable_name
+                        params_stack.append(
+                            _CxSetParamFromValue(
+                                field_name, cx_converter.field_variable_name
+                            )
                         )
-                        params_stack.append(setter)
 
-                        parent.append_child(construct)
-                        parent = construct
+                        parent.append_child(cx_converter)
+                        parent = cx_converter
                     else:
-                        params_stack.append(_CxSetParamFromPath(node.var_name, level))
+                        params_stack.append(_CxSetParamFromPath(field_name, level))
 
                     # NOTE(kgriffs): We don't allow multiple simple var nodes
                     # to exist at the same level, e.g.:
@@ -503,9 +561,9 @@ class CompiledRouter:
 
             else:
                 # NOTE(kgriffs): Not a param, so must match exactly
-                construct = _CxIfPathSegmentLiteral(level, node.raw_segment)
-                parent.append_child(construct)
-                parent = construct
+                cx_literal = _CxIfPathSegmentLiteral(level, node.raw_segment)
+                parent.append_child(cx_literal)
+                parent = cx_literal
 
             if node.resource is not None:
                 # NOTE(kgriffs): This is a valid route, so we will want to
@@ -513,6 +571,8 @@ class CompiledRouter:
                 resource_idx = len(return_values)
                 return_values.append(node)
 
+            assert not (consume_multiple_segments and node.children)
+
             self._generate_ast(
                 node.children,
                 parent,
@@ -527,17 +587,22 @@ class CompiledRouter:
                 if fast_return:
                     parent.append_child(_CxReturnNone())
             else:
-                # NOTE(kgriffs): Make sure that we have consumed all of
-                # the segments for the requested route; otherwise we could
-                # mistakenly match "/foo/23/bar" against "/foo/{id}".
-                construct = _CxIfPathLength('==', level + 1)
-                for params in params_stack:
-                    construct.append_child(params)
-                construct.append_child(_CxReturnValue(resource_idx))
-                parent.append_child(construct)
+                if consume_multiple_segments:
+                    for params in params_stack:
+                        parent.append_child(params)
+                    parent.append_child(_CxReturnValue(resource_idx))
+                else:
+                    # NOTE(kgriffs): Make sure that we have consumed all of
+                    # the segments for the requested route; otherwise we could
+                    # mistakenly match "/foo/23/bar" against "/foo/{id}".
+                    cx_path_len = _CxIfPathLength('==', level + 1)
+                    for params in params_stack:
+                        cx_path_len.append_child(params)
+                    cx_path_len.append_child(_CxReturnValue(resource_idx))
+                    parent.append_child(cx_path_len)
 
-                if fast_return:
-                    parent.append_child(_CxReturnNone())
+                    if fast_return:
+                        parent.append_child(_CxReturnNone())
 
             parent = outer_parent
 
@@ -545,14 +610,16 @@ class CompiledRouter:
             parent.append_child(_CxReturnNone())
 
     def _generate_conversion_ast(
-        self, parent, node: 'CompiledRouterNode', params_stack: list
-    ):
-        construct = None  # type: Any
-        setter = None  # type: Any
+        self,
+        parent: _CxParent,
+        node: CompiledRouterNode,
+        params_stack: List[_CxElement],
+    ) -> _CxParent:
         # NOTE(kgriffs): Unroll the converter loop into
         # a series of nested "if" constructs.
         for field_name, converter_name, converter_argstr in node.var_converter_map:
             converter_class = self._converter_map[converter_name]
+            assert not converters._consumes_multiple_segments(converter_class)
 
             converter_obj = self._instantiate_converter(
                 converter_class, converter_argstr
@@ -562,24 +629,28 @@ class CompiledRouter:
 
             parent.append_child(_CxSetFragmentFromField(field_name))
 
-            construct = _CxIfConverterField(len(params_stack) + 1, converter_idx)
-            setter = _CxSetParamFromValue(field_name, construct.field_variable_name)
-            params_stack.append(setter)
+            cx_converter = _CxIfConverterField(len(params_stack) + 1, converter_idx)
+            params_stack.append(
+                _CxSetParamFromValue(field_name, cx_converter.field_variable_name)
+            )
 
-            parent.append_child(construct)
-            parent = construct
+            parent.append_child(cx_converter)
+            parent = cx_converter
 
         # NOTE(kgriffs): Add remaining fields that were not
         # converted, if any.
         if node.num_fields > len(node.var_converter_map):
-            construct = _CxVariableFromPatternMatchPrefetched(len(params_stack) + 1)
-            setter = _CxSetParamsFromDict(construct.dict_variable_name)
-            params_stack.append(setter)
-            parent.append_child(construct)
+            cx_pattern_match = _CxVariableFromPatternMatchPrefetched(
+                len(params_stack) + 1
+            )
+            params_stack.append(
+                _CxSetParamsFromDict(cx_pattern_match.dict_variable_name)
+            )
+            parent.append_child(cx_pattern_match)
 
         return parent
 
-    def _compile(self):
+    def _compile(self) -> Callable:
         """Generate Python code for the entire routing tree.
 
         The generated code is compiled and the resulting Python method
@@ -602,20 +673,19 @@ class CompiledRouter:
 
         src_lines.append(self._ast.src(0))
 
-        src_lines.append(
-            # PERF(kgriffs): Explicit return of None is faster than implicit
-            _TAB_STR
-            + 'return None'
-        )
+        # PERF(kgriffs): Explicit return of None is faster than implicit
+        src_lines.append(_TAB_STR + 'return None')
 
         self._finder_src = '\n'.join(src_lines)
 
-        scope = {}
+        scope: MethodDict = {}
         exec(compile(self._finder_src, '<string>', 'exec'), scope)
 
         return scope['find']
 
-    def _instantiate_converter(self, klass, argstr=None):
+    def _instantiate_converter(
+        self, klass: type, argstr: Optional[str] = None
+    ) -> converters.BaseConverter:
         if argstr is None:
             return klass()
 
@@ -623,7 +693,14 @@ class CompiledRouter:
         src = '{0}({1})'.format(klass.__name__, argstr)
         return eval(src, {klass.__name__: klass})
 
-    def _compile_and_find(self, path, _return_values, _patterns, _converters, params):
+    def _compile_and_find(
+        self,
+        path: List[str],
+        _return_values: Any,
+        _patterns: Any,
+        _converters: Any,
+        params: Any,
+    ) -> Any:
         """Compile the router, set the `_find` attribute and return its result.
 
         This method is set to the `_find` attribute to delay the compilation of the
@@ -644,11 +721,28 @@ class CompiledRouter:
         )
 
 
+_NO_CHILDREN_ERR = (
+    'Cannot add route with template "{0}". Field name "{1}" '
+    'uses the converter "{2}" that will consume all the path, '
+    'making it impossible to match this route.'
+)
+
+
+class UnacceptableRouteError(ValueError):
+    """Error raised by ``add_error`` when a route is not acceptable."""
+
+
 class CompiledRouterNode:
     """Represents a single URI segment in a URI."""
 
-    def __init__(self, raw_segment, method_map=None, resource=None, uri_template=None):
-        self.children = []
+    def __init__(
+        self,
+        raw_segment: str,
+        method_map: Optional[MethodDict] = None,
+        resource: Optional[object] = None,
+        uri_template: Optional[str] = None,
+    ) -> None:
+        self.children: List[CompiledRouterNode] = []
 
         self.raw_segment = raw_segment
         self.method_map = method_map
@@ -661,9 +755,9 @@ class CompiledRouterNode:
 
         # TODO(kgriffs): Rename these since the docs talk about "fields"
         # or "field expressions", not "vars" or "variables".
-        self.var_name = None
-        self.var_pattern = None
-        self.var_converter_map = []
+        self.var_name: Optional[str] = None
+        self.var_pattern: Optional[Pattern] = None
+        self.var_converter_map: List[Tuple[str, str, str]] = []
 
         # NOTE(kgriffs): CompiledRouter.add_route validates field names,
         # so here we can just assume they are OK and use the simple
@@ -735,12 +829,12 @@ class CompiledRouterNode:
         if self.is_complex:
             assert self.is_var
 
-    def matches(self, segment):
+    def matches(self, segment: str) -> bool:
         """Return True if this node matches the supplied template segment."""
 
         return segment == self.raw_segment
 
-    def conflicts_with(self, segment):
+    def conflicts_with(self, segment: str) -> bool:
         """Return True if this node conflicts with a given template segment."""
 
         # NOTE(kgriffs): This method assumes that the caller has already
@@ -800,29 +894,13 @@ class CompiledRouterNode:
 class ConverterDict(UserDict):
     """A dict-like class for storing field converters."""
 
-    def update(self, other):
-        try:
-            # NOTE(kgriffs): If it is a mapping type, it should
-            # implement keys().
-            names = other.keys()
-        except AttributeError:
-            # NOTE(kgriffs): Not a mapping type, so assume it is an
-            # iterable of 2-item iterables. But we need to make it
-            # re-iterable if it is a generator, for when we pass
-            # it on to the parent's update().
-            other = list(other)
-            names = [n for n, __ in other]
+    data: Dict[str, Type[converters.BaseConverter]]
 
-        for n in names:
-            self._validate(n)
-
-        UserDict.update(self, other)
-
-    def __setitem__(self, name, converter):
+    def __setitem__(self, name: str, converter: Type[converters.BaseConverter]) -> None:
         self._validate(name)
         UserDict.__setitem__(self, name, converter)
 
-    def _validate(self, name):
+    def _validate(self, name: str) -> None:
         if not _IDENTIFIER_PATTERN.match(name):
             raise ValueError(
                 'Invalid converter name. Names may not be blank, and may '
@@ -834,44 +912,50 @@ class ConverterDict(UserDict):
 class CompiledRouterOptions:
     """Defines a set of configurable router options.
 
-    An instance of this class is exposed via :py:attr:`falcon.App.router_options`
-    and :py:attr:`falcon.asgi.App.router_options` for configuring certain
-    :py:class:`~.CompiledRouter` behaviors.
+    An instance of this class is exposed via :attr:`falcon.App.router_options`
+    and :attr:`falcon.asgi.App.router_options` for configuring certain
+    :class:`~.CompiledRouter` behaviors.
+    """
 
-    Attributes:
-        converters: Represents the collection of named
-            converters that may be referenced in URI template field
-            expressions. Adding additional converters is simply a
-            matter of mapping an identifier to a converter class::
+    converters: ConverterDict
+    """Represents the collection of named converters that may
+    be referenced in URI template field expressions.
 
-                app.router_options.converters['mc'] = MyConverter
+    Adding additional converters is simply a matter of mapping an identifier to
+    a converter class::
 
-            The identifier can then be used to employ the converter
-            within a URI template::
+        app.router_options.converters['mc'] = MyConverter
 
-                app.add_route('/{some_field:mc}', some_resource)
+    The identifier can then be used to employ the converter within a URI template::
 
-            Converter names may only contain ASCII letters, digits,
-            and underscores, and must start with either a letter or
-            an underscore.
+        app.add_route('/{some_field:mc}', some_resource)
 
-            Warning:
+    Converter names may only contain ASCII letters, digits, and underscores, and
+    must start with either a letter or an underscore.
 
-                Converter instances are shared between requests.
-                Therefore, in threaded deployments, care must be taken
-                to implement custom converters in a thread-safe
-                manner.
+    Warning:
 
-            (See also: :ref:`Field Converters <routing_field_converters>`)
+        Converter instances are shared between requests.
+        Therefore, in threaded deployments, care must be taken to implement custom
+        converters in a thread-safe manner.
+
+    (See also: :ref:`Field Converters <routing_field_converters>`)
     """
 
     __slots__ = ('converters',)
 
-    def __init__(self):
-        self.converters = ConverterDict(
-            (name, converter) for name, converter in converters.BUILTIN
+    def __init__(self) -> None:
+        object.__setattr__(
+            self,
+            'converters',
+            ConverterDict((name, converter) for name, converter in converters.BUILTIN),
         )
 
+    def __setattr__(self, name: str, value: Any) -> None:
+        if name == 'converters':
+            raise AttributeError('Cannot set "converters", please update it in place.')
+        super().__setattr__(name, value)
+
 
 # --------------------------------------------------------------------
 # AST Constructs
@@ -887,28 +971,34 @@ class CompiledRouterOptions:
 
 
 class _CxParent:
-    def __init__(self):
-        self._children = []
+    def __init__(self) -> None:
+        self._children: List[_CxElement] = []
 
-    def append_child(self, construct):
+    def append_child(self, construct: _CxElement) -> None:
         self._children.append(construct)
 
-    def src(self, indentation):
+    def src(self, indentation: int) -> str:
         return self._children_src(indentation + 1)
 
-    def _children_src(self, indentation):
+    def _children_src(self, indentation: int) -> str:
         src_lines = [child.src(indentation) for child in self._children]
 
         return '\n'.join(src_lines)
 
 
+class _CxChild:
+    # This a base element only to aid pep484
+    def src(self, indentation: int) -> str:
+        raise NotImplementedError
+
+
 class _CxIfPathLength(_CxParent):
-    def __init__(self, comparison, length):
+    def __init__(self, comparison: str, length: int) -> None:
         super().__init__()
         self._comparison = comparison
         self._length = length
 
-    def src(self, indentation):
+    def src(self, indentation: int) -> str:
         template = '{0}if path_len {1} {2}:\n{3}'
         return template.format(
             _TAB_STR * indentation,
@@ -919,12 +1009,12 @@ class _CxIfPathLength(_CxParent):
 
 
 class _CxIfPathSegmentLiteral(_CxParent):
-    def __init__(self, segment_idx, literal):
+    def __init__(self, segment_idx: int, literal: str) -> None:
         super().__init__()
         self._segment_idx = segment_idx
         self._literal = literal
 
-    def src(self, indentation):
+    def src(self, indentation: int) -> str:
         template = "{0}if path[{1}] == '{2}':\n{3}"
         return template.format(
             _TAB_STR * indentation,
@@ -935,13 +1025,13 @@ class _CxIfPathSegmentLiteral(_CxParent):
 
 
 class _CxIfPathSegmentPattern(_CxParent):
-    def __init__(self, segment_idx, pattern_idx, pattern_text):
+    def __init__(self, segment_idx: int, pattern_idx: int, pattern_text: str) -> None:
         super().__init__()
         self._segment_idx = segment_idx
         self._pattern_idx = pattern_idx
         self._pattern_text = pattern_text
 
-    def src(self, indentation):
+    def src(self, indentation: int) -> str:
         lines = [
             '{0}match = patterns[{1}].match(path[{2}])  # {3}'.format(
                 _TAB_STR * indentation,
@@ -957,13 +1047,13 @@ class _CxIfPathSegmentPattern(_CxParent):
 
 
 class _CxIfConverterField(_CxParent):
-    def __init__(self, unique_idx, converter_idx):
+    def __init__(self, unique_idx: int, converter_idx: int) -> None:
         super().__init__()
         self._converter_idx = converter_idx
         self._unique_idx = unique_idx
         self.field_variable_name = 'field_value_{0}'.format(unique_idx)
 
-    def src(self, indentation):
+    def src(self, indentation: int) -> str:
         lines = [
             '{0}{1} = converters[{2}].convert(fragment)'.format(
                 _TAB_STR * indentation,
@@ -979,74 +1069,85 @@ class _CxIfConverterField(_CxParent):
         return '\n'.join(lines)
 
 
-class _CxSetFragmentFromField:
-    def __init__(self, field_name):
+class _CxSetFragmentFromField(_CxChild):
+    def __init__(self, field_name: str) -> None:
         self._field_name = field_name
 
-    def src(self, indentation):
+    def src(self, indentation: int) -> str:
         return "{0}fragment = groups.pop('{1}')".format(
             _TAB_STR * indentation,
             self._field_name,
         )
 
 
-class _CxSetFragmentFromPath:
-    def __init__(self, segment_idx):
+class _CxSetFragmentFromPath(_CxChild):
+    def __init__(self, segment_idx: int) -> None:
         self._segment_idx = segment_idx
 
-    def src(self, indentation):
+    def src(self, indentation: int) -> str:
         return '{0}fragment = path[{1}]'.format(
             _TAB_STR * indentation,
             self._segment_idx,
         )
 
 
-class _CxVariableFromPatternMatch:
-    def __init__(self, unique_idx):
+class _CxSetFragmentFromRemainingPaths(_CxChild):
+    def __init__(self, segment_idx: int) -> None:
+        self._segment_idx = segment_idx
+
+    def src(self, indentation: int) -> str:
+        return '{0}fragment = path[{1}:]'.format(
+            _TAB_STR * indentation,
+            self._segment_idx,
+        )
+
+
+class _CxVariableFromPatternMatch(_CxChild):
+    def __init__(self, unique_idx: int) -> None:
         self._unique_idx = unique_idx
         self.dict_variable_name = 'dict_match_{0}'.format(unique_idx)
 
-    def src(self, indentation):
+    def src(self, indentation: int) -> str:
         return '{0}{1} = match.groupdict()'.format(
             _TAB_STR * indentation, self.dict_variable_name
         )
 
 
-class _CxVariableFromPatternMatchPrefetched:
-    def __init__(self, unique_idx):
+class _CxVariableFromPatternMatchPrefetched(_CxChild):
+    def __init__(self, unique_idx: int) -> None:
         self._unique_idx = unique_idx
         self.dict_variable_name = 'dict_groups_{0}'.format(unique_idx)
 
-    def src(self, indentation):
+    def src(self, indentation: int) -> str:
         return '{0}{1} = groups'.format(_TAB_STR * indentation, self.dict_variable_name)
 
 
-class _CxPrefetchGroupsFromPatternMatch:
-    def src(self, indentation):
+class _CxPrefetchGroupsFromPatternMatch(_CxChild):
+    def src(self, indentation: int) -> str:
         return '{0}groups = match.groupdict()'.format(_TAB_STR * indentation)
 
 
-class _CxReturnNone:
-    def src(self, indentation):
+class _CxReturnNone(_CxChild):
+    def src(self, indentation: int) -> str:
         return '{0}return None'.format(_TAB_STR * indentation)
 
 
-class _CxReturnValue:
-    def __init__(self, value_idx):
+class _CxReturnValue(_CxChild):
+    def __init__(self, value_idx: int) -> None:
         self._value_idx = value_idx
 
-    def src(self, indentation):
+    def src(self, indentation: int) -> str:
         return '{0}return return_values[{1}]'.format(
             _TAB_STR * indentation, self._value_idx
         )
 
 
-class _CxSetParamFromPath:
-    def __init__(self, param_name, segment_idx):
+class _CxSetParamFromPath(_CxChild):
+    def __init__(self, param_name: str, segment_idx: int) -> None:
         self._param_name = param_name
         self._segment_idx = segment_idx
 
-    def src(self, indentation):
+    def src(self, indentation: int) -> str:
         return "{0}params['{1}'] = path[{2}]".format(
             _TAB_STR * indentation,
             self._param_name,
@@ -1054,12 +1155,12 @@ class _CxSetParamFromPath:
         )
 
 
-class _CxSetParamFromValue:
-    def __init__(self, param_name, field_value_name):
+class _CxSetParamFromValue(_CxChild):
+    def __init__(self, param_name: str, field_value_name: str) -> None:
         self._param_name = param_name
         self._field_value_name = field_value_name
 
-    def src(self, indentation):
+    def src(self, indentation: int) -> str:
         return "{0}params['{1}'] = {2}".format(
             _TAB_STR * indentation,
             self._param_name,
@@ -1067,11 +1168,11 @@ class _CxSetParamFromValue:
         )
 
 
-class _CxSetParamsFromDict:
-    def __init__(self, dict_value_name):
+class _CxSetParamsFromDict(_CxChild):
+    def __init__(self, dict_value_name: str) -> None:
         self._dict_value_name = dict_value_name
 
-    def src(self, indentation):
+    def src(self, indentation: int) -> str:
         return '{0}params.update({1})'.format(
             _TAB_STR * indentation,
             self._dict_value_name,
