@@ -108,7 +108,6 @@ def test_basic_aiter(reader1):
     ]
 
 
-@falcon.runs_sync
 async def test_aiter_from_buffer(reader1):
     assert await reader1.read(4) == b'Hell'
 
@@ -148,27 +147,23 @@ def test_delimit(reader1, delimiter, expected):
     assert async_take(delimited) == expected
 
 
-@falcon.runs_sync
 async def test_exhaust(reader1):
     await reader1.exhaust()
     assert await reader1.peek() == b''
 
 
 @pytest.mark.parametrize('size', [1, 2, 3, 5, 7, 8])
-@falcon.runs_sync
 async def test_peek(reader1, size):
     assert await reader1.peek(size) == b'Hello, World'[:size]
     assert reader1.tell() == 0
 
 
-@falcon.runs_sync
 async def test_peek_at_eof():
     source = chop_data(b'Hello!')
     stream = reader.BufferedReader(source)
     assert await stream.peek(16) == b'Hello!'
 
 
-@falcon.runs_sync
 async def test_pipe(reader1):
     sink = AsyncSink()
     await reader1.pipe(sink)
@@ -177,7 +172,6 @@ async def test_pipe(reader1):
     assert reader1.tell() == len(sink.accumulated)
 
 
-@falcon.runs_sync
 async def test_pipe_until_delimiter_not_found(reader1):
     sink = AsyncSink()
     await reader1.pipe_until(b'404', sink)
@@ -202,7 +196,6 @@ async def test_pipe_until_delimiter_not_found(reader1):
         ((50, 1), [b'Hello, World!\nJust testing some iterator goodness.', b'\n']),
     ],
 )
-@falcon.runs_sync
 async def test_read(reader1, sizes, expected):
     results = []
     for size in sizes:
@@ -212,7 +205,7 @@ async def test_read(reader1, sizes, expected):
 
 
 @pytest.mark.parametrize('start_size', [1, 16777216])
-@falcon.runs_sync
+@pytest.mark.slow
 async def test_varying_read_size(reader2, start_size):
     size = start_size
     result = io.BytesIO()
@@ -229,7 +222,6 @@ async def test_varying_read_size(reader2, start_size):
 
 
 @pytest.mark.parametrize('peek', [0, 1, 8])
-@falcon.runs_sync
 async def test_readall(reader1, peek):
     if peek:
         await reader1.peek(peek)
@@ -262,7 +254,6 @@ async def test_readall(reader1, peek):
         (7, b'good', -1, b'World!\nJust testing some iterator '),
     ],
 )
-@falcon.runs_sync
 async def test_read_until(reader1, offset, delimiter, size, expected, fork):
     if offset:
         await reader1.read(offset)
@@ -273,7 +264,6 @@ async def test_read_until(reader1, offset, delimiter, size, expected, fork):
         assert await reader1.read_until(delimiter, size) == expected
 
 
-@falcon.runs_sync
 async def test_read_until_with_buffer_edge_case(reader1):
     assert await reader1.read(12) == b'Hello, World'
     assert await reader1.peek(1) == b'!'
@@ -291,7 +281,6 @@ def test_placeholder_methods(reader1):
     assert not reader1.writable()
 
 
-@falcon.runs_sync
 async def test_iteration_started(reader1):
     async for chunk in reader1:
         with pytest.raises(OperationNotAllowed):
@@ -299,7 +288,6 @@ async def test_iteration_started(reader1):
                 pass
 
 
-@falcon.runs_sync
 async def test_invalid_delimiter_length(reader1):
     with pytest.raises(ValueError):
         await reader1.read_until(b'')
@@ -318,7 +306,7 @@ async def test_invalid_delimiter_length(reader1):
         (13372477, 51637898),
     ],
 )
-@falcon.runs_sync
+@pytest.mark.slow
 async def test_irregular_large_read_until(reader2, size1, size2):
     delimiter = b'--boundary1234567890--'
 
@@ -339,7 +327,6 @@ async def test_irregular_large_read_until(reader2, size1, size2):
     assert chunk1 + chunk2 + remainder == expected[1337:]
 
 
-@falcon.runs_sync
 @pytest.mark.parametrize('chunk_size', list(range(46, 63)))
 async def test_read_until_shared_boundary(chunk_size):
     source = chop_data(
@@ -354,7 +341,8 @@ async def test_read_until_shared_boundary(chunk_size):
 
 # NOTE(vytas): This is woefully unoptimized, and this test highlights that.
 #   Work in progress.
-@falcon.runs_sync
+
+
 async def test_small_reads(reader3):
     ops = 0
     read = 0
@@ -376,7 +364,7 @@ async def test_small_reads(reader3):
     assert last.endswith(b'4')
 
 
-@falcon.runs_sync
+@pytest.mark.slow
 async def test_small_reads_with_delimiter(reader3):
     ops = 0
     read = 0
