@@ -2,15 +2,16 @@ import io
 import os
 import tempfile
 
-import aiofiles
 import pytest
 
 import falcon
 from falcon import testing
 import falcon.asgi
 
-from _util import disable_asgi_non_coroutine_wrapping  # NOQA
-
+try:
+    import aiofiles
+except ImportError:
+    aiofiles = None  # type: ignore[assignment]
 
 SIZE_1_KB = 1024
 
@@ -310,6 +311,7 @@ class TestHelloWorld:
         if assert_closed:
             assert resource.stream.close_called
 
+    @pytest.mark.skipif(aiofiles is None, reason='aiofiles is required for this test')
     def test_filelike_closing_aiofiles(self, client):
         resource = AIOFilesHelloResource()
         try:
@@ -373,8 +375,8 @@ class TestHelloWorld:
         assert not result.content
         assert result.status_code == 200
 
-    def test_coroutine_required(self, client):
-        with disable_asgi_non_coroutine_wrapping():
+    def test_coroutine_required(self, client, util):
+        with util.disable_asgi_non_coroutine_wrapping():
             with pytest.raises(TypeError) as exinfo:
                 client.app.add_route('/', PartialCoroutineResource())
 
