@@ -4,11 +4,11 @@ import time
 
 import pytest
 
-from falcon import runs_sync
 from falcon import testing
 from falcon.asgi import App
 
 
+@pytest.mark.slow
 def test_multiple():
     class SomeResource:
         def __init__(self):
@@ -71,7 +71,7 @@ def test_multiple():
     assert result.status_code == 500
 
     # NOTE(kgriffs): Remove default handlers so that we can check the raised
-    #   exception is what we expecte.
+    #   exception is what we expected.
     app._error_handlers.clear()
     with pytest.raises(TypeError) as exinfo:
         client.simulate_put()
@@ -137,7 +137,6 @@ def callback_app(simple_resource):
         ('GET', '/stream', 'One\nTwo\nThree\n'),
     ],
 )
-@runs_sync
 async def test_callback(callback_app, simple_resource, method, uri, expected):
     async with testing.ASGIConductor(callback_app) as conductor:
         resp = await conductor.simulate_request(method, uri)
