@@ -1,31 +1,37 @@
 import asyncio
 from collections import deque
+import functools
 import os
 
-import cbor2
 import pytest
 
-
 import falcon
-from falcon import media, testing
+from falcon import media
+from falcon import testing
 from falcon.asgi import App
 from falcon.asgi.ws import _WebSocketState as ServerWebSocketState
+from falcon.asgi.ws import WebSocket
 from falcon.asgi.ws import WebSocketOptions
 from falcon.testing.helpers import _WebSocketState as ClientWebSocketState
 
-
 try:
-    import rapidjson  # type: ignore
+    import cbor2
 except ImportError:
-    rapidjson = None
+    cbor2 = None  # type: ignore[assignment]
 
 
 try:
-    import msgpack  # type: ignore
+    import msgpack
 except ImportError:
     msgpack = None
 
 
+try:
+    import rapidjson
+except ImportError:
+    rapidjson = None  # type: ignore[assignment]
+
+
 # NOTE(kgriffs): We do not use codes defined in the framework because we
 #   want to verify that the correct value is being used.
 class CloseCode:
@@ -50,7 +56,6 @@ def conductor():
     return testing.ASGIConductor(app)
 
 
-@pytest.mark.asyncio
 @pytest.mark.parametrize('path', ['/ws/yes', '/ws/no'])
 async def test_ws_not_accepted(path, conductor):
     class SomeResource:
@@ -108,7 +113,7 @@ async def test_ws_not_accepted(path, conductor):
             assert resource.caught_operation_not_allowed
 
 
-@pytest.mark.asyncio
+@pytest.mark.slow
 async def test_echo():  # noqa: C901
     consumer_sleep = 0.01
     producer_loop = 10
@@ -141,7 +146,7 @@ async def test_echo():  # noqa: C901
                 await ws.send_text(f'{p1}:{p2}:{req.context.message}:{injected}')
 
             messages = deque()
-            sink_task = falcon.create_task(self._sink(ws, messages))
+            sink_task = asyncio.create_task(self._sink(ws, messages))
 
             while not sink_task.done():
                 if not messages:
@@ -226,7 +231,6 @@ async def test_echo():  # noqa: C901
     assert resource.caught_operation_not_allowed
 
 
-@pytest.mark.asyncio
 async def test_path_not_found(conductor):
     async with conductor as c:
         with pytest.raises(falcon.WebSocketDisconnected) as exc_info:
@@ -236,7 +240,6 @@ async def test_path_not_found(conductor):
         assert exc_info.value.code == CloseCode.NOT_FOUND
 
 
-@pytest.mark.asyncio
 @pytest.mark.parametrize('clear_error_handlers', [True, False])
 async def test_responder_raises_unhandled_error(clear_error_handlers, conductor):
     class SomeResource:
@@ -296,7 +299,6 @@ async def test_responder_raises_unhandled_error(clear_error_handlers, conductor)
             assert exc_info.value.code == 3422
 
 
-@pytest.mark.asyncio
 @pytest.mark.parametrize('direction', ['send', 'receive'])
 @pytest.mark.parametrize('explicit_close_client', [True, False])
 @pytest.mark.parametrize('explicit_close_server', [True, False])
@@ -338,13 +340,12 @@ async def test_client_disconnect_early(  # noqa: C901
                             #   order to test coverage of the logic that handles
                             #   the case of a closed connection while waiting on
                             #   more data.
-                            recv_task = falcon.create_task(ws.receive_data())
-                            await asyncio.sleep(
-                                0
-                            )  # Ensure recv_task() has a chance to get ahead
-                            await asyncio.wait(
-                                [recv_task, falcon.create_task(ws.close(4099))]
-                            )
+                            recv_task = asyncio.create_task(ws.receive_data())
+                            # Ensure recv_task() has a chance to get ahead
+                            await asyncio.sleep(0)
+                            ws_close = ws.close(4099)
+                            ws_close = asyncio.create_task(ws_close)
+                            await asyncio.wait([recv_task, ws_close])
 
                         self.data_received.set()
 
@@ -371,6 +372,13 @@ async def test_client_disconnect_early(  # noqa: C901
                     await resource.data_received.wait()
                     assert resource.data == sample_data
 
+                # NOTE(vytas): When testing the case where the server
+                #   explicitly closes the connection, try to receive some data
+                #   before closing from the client side (and potentially
+                #   winning the async race of which side closes first).
+                if explicit_close_server:
+                    await ws.receive_data()
+
                 if explicit_close_client:
                     await ws.close(4042)
 
@@ -398,9 +406,9 @@ async def test_client_disconnect_early(  # noqa: C901
     assert resource.ws_ready is False
 
 
-@pytest.mark.asyncio
 @pytest.mark.parametrize('custom_text', [True, False])
 @pytest.mark.parametrize('custom_data', [True, False])
+@pytest.mark.skipif(msgpack is None, reason='msgpack is required for this test')
 async def test_media(custom_text, custom_data, conductor):  # NOQA: C901
     # TODO(kgriffs): Refactor to reduce McCabe score
 
@@ -435,7 +443,7 @@ async def test_media(custom_text, custom_data, conductor):  # NOQA: C901
                 for __ in range(3):
                     try:
                         await ws.receive_media()
-                    except (ValueError):
+                    except ValueError:
                         self.deserialize_error_count += 1
             finally:
                 self.finished.set()
@@ -460,11 +468,13 @@ async def test_media(custom_text, custom_data, conductor):  # NOQA: C901
             def deserialize(self, payload: str) -> object:
                 return rapidjson.loads(payload)
 
-        app.ws_options.media_handlers[
-            falcon.WebSocketPayloadType.TEXT
-        ] = RapidJSONHandler()
+        app.ws_options.media_handlers[falcon.WebSocketPayloadType.TEXT] = (
+            RapidJSONHandler()
+        )
 
     if custom_data:
+        if cbor2 is None:
+            pytest.skip('cbor2 is required for this test')
 
         class CBORHandler(media.BinaryBaseHandlerWS):
             def serialize(self, media: object) -> bytes:
@@ -474,9 +484,9 @@ async def test_media(custom_text, custom_data, conductor):  # NOQA: C901
             def deserialize(self, payload: bytes) -> object:
                 return cbor2.loads(payload)
 
-        app.ws_options.media_handlers[
-            falcon.WebSocketPayloadType.BINARY
-        ] = CBORHandler()
+        app.ws_options.media_handlers[falcon.WebSocketPayloadType.BINARY] = (
+            CBORHandler()
+        )
 
     async with conductor as c:
         async with c.simulate_ws() as ws:
@@ -496,7 +506,7 @@ async def test_media(custom_text, custom_data, conductor):  # NOQA: C901
             #    ensure we aren't getting any false-positives.
             await ws.send_text('"DEADBEEF"')
             await ws.send_text('DEADBEEF')
-            await ws.send_data(b'\xDE\xAD\xBE\xEF')
+            await ws.send_data(b'\xde\xad\xbe\xef')
 
             await resource.finished.wait()
 
@@ -510,7 +520,6 @@ async def test_media(custom_text, custom_data, conductor):  # NOQA: C901
         assert doc == doc_expected
 
 
-@pytest.mark.asyncio
 @pytest.mark.parametrize('sample_data', [b'123', b'', b'\xe1\x9a\xa0\xe1', b'\0'])
 async def test_send_receive_data(sample_data, conductor):
     class Resource:
@@ -552,7 +561,6 @@ async def test_send_receive_data(sample_data, conductor):
     assert resource.error_count == 4
 
 
-@pytest.mark.asyncio
 @pytest.mark.parametrize(
     'subprotocols',
     [
@@ -590,7 +598,6 @@ async def test_subprotocol(subprotocols, conductor):
                 resource.test_complete.set()
 
 
-@pytest.mark.asyncio
 @pytest.mark.parametrize(
     'headers',
     [
@@ -639,7 +646,6 @@ async def test_accept_with_headers(headers, conductor):
                 resource.test_complete.set()
 
 
-@pytest.mark.asyncio
 @pytest.mark.parametrize(
     'headers',
     [
@@ -673,7 +679,6 @@ async def test_accept_with_bad_headers(headers, conductor):
     assert isinstance(resource.raised_error, ValueError)
 
 
-@pytest.mark.asyncio
 async def test_accept_with_headers_not_supported(conductor):
     class Resource:
         def __init__(self):
@@ -699,7 +704,6 @@ async def test_accept_with_headers_not_supported(conductor):
     assert isinstance(resource.raised_error, falcon.OperationNotAllowed)
 
 
-@pytest.mark.asyncio
 async def test_missing_ws_handler(conductor):
     class Resource:
         async def on_get(self, req, resp):
@@ -713,7 +717,6 @@ async def test_missing_ws_handler(conductor):
                 pass
 
 
-@pytest.mark.asyncio
 async def test_unexpected_param(conductor):
     class Resource:
         async def on_websocket(self, req, ws):
@@ -727,7 +730,6 @@ async def test_unexpected_param(conductor):
                 pass
 
 
-@pytest.mark.asyncio
 @pytest.mark.parametrize(
     'subprotocol',
     [
@@ -760,7 +762,6 @@ async def test_subprotocol_bad_type(subprotocol, conductor):
                     pass
 
 
-@pytest.mark.asyncio
 async def test_send_receive_wrong_type(conductor):
     class Resource:
         def __init__(self):
@@ -815,7 +816,6 @@ async def test_send_receive_wrong_type(conductor):
     assert resource.error_count == 4
 
 
-@pytest.mark.asyncio
 @pytest.mark.parametrize(
     'options_code',
     [999, 100, 0, -1, 1004, 1005, 1006, 1015, 1016, 1017, 1050, 1099, 'NaN'],
@@ -868,7 +868,6 @@ def test_mw_methods_must_be_coroutines():
             App(middleware=mw)
 
 
-@pytest.mark.asyncio
 @pytest.mark.parametrize('version', ['1.9', '20.5', '3.0', '3.1'])
 async def test_bad_spec_version(version, conductor):
     async with conductor as c:
@@ -877,7 +876,6 @@ async def test_bad_spec_version(version, conductor):
                 pass
 
 
-@pytest.mark.asyncio
 @pytest.mark.parametrize('version', ['1.0', '1'])
 async def test_bad_http_version(version, conductor):
     async with conductor as c:
@@ -886,12 +884,11 @@ async def test_bad_http_version(version, conductor):
                 pass
 
 
-@pytest.mark.asyncio
-async def test_bad_first_event():
+@pytest.mark.parametrize('version', ['2.1', '2.3', '2.10.3'])
+async def test_bad_first_event(version):
     app = App()
 
-    scope = testing.create_scope_ws()
-    del scope['asgi']['spec_version']
+    scope = testing.create_scope_ws(spec_version=version)
 
     ws = testing.ASGIWebSocketSimulator()
     wrapped_emit = ws._emit
@@ -911,9 +908,12 @@ async def test_bad_first_event():
 
     assert ws.closed
     assert ws.close_code == CloseCode.SERVER_ERROR
+    if version != '2.1':
+        assert ws.close_reason == 'Internal Server Error'
+    else:
+        assert ws.close_reason == ''
 
 
-@pytest.mark.asyncio
 async def test_missing_http_version():
     app = App()
 
@@ -927,7 +927,6 @@ async def test_missing_http_version():
     await app(scope, ws._emit, ws._collect)
 
 
-@pytest.mark.asyncio
 async def test_missing_spec_version():
     app = App()
 
@@ -941,7 +940,6 @@ async def test_missing_spec_version():
     await app(scope, ws._emit, ws._collect)
 
 
-@pytest.mark.asyncio
 async def test_translate_webserver_error(conductor):
     class Resource:
         def __init__(self):
@@ -951,36 +949,77 @@ async def test_translate_webserver_error(conductor):
         async def on_websocket(self, req, ws):
             await ws.accept()
 
-            async def raise_disconnect(self):
+            async def raise_disconnect(event):
                 raise Exception('Disconnected with code = 1000 (OK)')
 
-            async def raise_protocol_mismatch(self):
+            async def raise_io_error(event, cause=''):
+                class ConnectionClosed(RuntimeError):
+                    pass
+
+                class ClientDisconnected(OSError):
+                    pass
+
+                if cause:
+                    raise ClientDisconnected() from ConnectionClosed(cause)
+                raise ClientDisconnected()
+
+            async def raise_protocol_mismatch(event):
                 raise Exception('protocol accepted must be from the list')
 
-            async def raise_other(self):
+            async def raise_other(event):
                 raise RuntimeError()
 
+            # TODO(vytas): It would be nice to somehow refactor this test not
+            #   to operate on the private members of WebSocket.
+            #   But, OTOH, it is quite useful as it is.
             _asgi_send = ws._asgi_send
+            _original_state = ws._state
 
             ws._asgi_send = raise_other
+            ws._state = _original_state
             try:
                 await ws.send_data(b'123')
             except Exception:
                 self.error_count += 1
 
             ws._asgi_send = raise_protocol_mismatch
+            ws._state = _original_state
             try:
                 await ws.send_data(b'123')
             except ValueError:
                 self.error_count += 1
 
             ws._asgi_send = raise_disconnect
+            ws._state = _original_state
             try:
                 await ws.send_data(b'123')
             except falcon.WebSocketDisconnected:
                 self.error_count += 1
 
+            ws._asgi_send = raise_io_error
+            ws._state = _original_state
+            try:
+                await ws.send_data(b'123')
+            except falcon.WebSocketDisconnected:
+                self.error_count += 1
+
+            ws._asgi_send = functools.partial(raise_io_error, cause='bork3d pipe')
+            ws._state = _original_state
+            try:
+                await ws.send_data(b'123')
+            except falcon.WebSocketDisconnected:
+                self.error_count += 1
+
+            ws._asgi_send = functools.partial(raise_io_error, cause='received 1001')
+            ws._state = _original_state
+            try:
+                await ws.send_data(b'123')
+            except falcon.WebSocketDisconnected:
+                self.error_count += 1
+            assert ws._close_code == 1001
+
             ws._asgi_send = _asgi_send
+            ws._state = _original_state
 
             self.test_complete.set()
 
@@ -991,6 +1030,8 @@ async def test_translate_webserver_error(conductor):
         async with c.simulate_ws():
             await resource.test_complete.wait()
 
+    assert resource.error_count == 6
+
 
 def test_ws_base_not_implemented():
     th = media.TextBaseHandlerWS()
@@ -1010,7 +1051,7 @@ def test_ws_base_not_implemented():
         bh.deserialize(b'')
 
 
-@pytest.mark.asyncio
+@pytest.mark.slow
 async def test_ws_context_timeout(conductor):
     class Resource:
         async def on_websocket(self, req, ws):
@@ -1024,7 +1065,6 @@ async def test_ws_context_timeout(conductor):
                 pass
 
 
-@pytest.mark.asyncio
 async def test_ws_simulator_client_require_accepted(conductor):
     class Resource:
         async def on_websocket(self, req, ws):
@@ -1043,7 +1083,6 @@ async def test_ws_simulator_client_require_accepted(conductor):
             await ws.receive_text()
 
 
-@pytest.mark.asyncio
 async def test_ws_simulator_collect_edge_cases(conductor):
     class Resource:
         pass
@@ -1051,7 +1090,7 @@ async def test_ws_simulator_collect_edge_cases(conductor):
     conductor.app.add_route('/', Resource())
 
     async with conductor as c:
-        context = c.simulate_ws()
+        context = c.simulate_ws(spec_version='2.3')
         ws = context._ws
 
         m = 'must receive the first websocket.connect'
@@ -1067,10 +1106,11 @@ async def test_ws_simulator_collect_edge_cases(conductor):
 
         await ws.close()
 
-        # NOTE(kgriffs): The collector should just eat all subsequent events
-        #   returned from the ASGI app.
-        for __ in range(100):
-            await ws._collect({'type': EventType.WS_SEND})
+        # NOTE(vytas): The collector should start raising an instance of
+        #   OSError from now on per the ASGI WS spec ver 2.4+.
+        for __ in range(10):
+            with pytest.raises(OSError):
+                await ws._collect({'type': EventType.WS_SEND})
 
         assert not ws._collected_server_events
 
@@ -1082,9 +1122,36 @@ async def test_ws_simulator_collect_edge_cases(conductor):
             event = await ws._emit()
 
 
+@pytest.mark.slow
+async def test_ws_responder_never_ready(conductor, monkeypatch):
+    async def noop_close(obj, code=None):
+        pass
+
+    class SleepyResource:
+        async def on_websocket(self, req, ws):
+            for i in range(10):
+                await asyncio.sleep(0.001)
+
+    conductor.app.add_route('/', SleepyResource())
+
+    # NOTE(vytas): It seems that it is hard to impossible to hit the second
+    #   `await ready_waiter` of the _WSContextManager on CPython 3.12 due to
+    #   different async code optimizations, so we mock away WebSocket.close.
+    monkeypatch.setattr(WebSocket, 'close', noop_close)
+
+    # NOTE(vytas): Shorten the timeout so that we do not wait for 5 seconds.
+    monkeypatch.setattr(
+        testing.ASGIWebSocketSimulator, '_DEFAULT_WAIT_READY_TIMEOUT', 0.5
+    )
+
+    async with conductor as c:
+        with pytest.raises(asyncio.TimeoutError):
+            async with c.simulate_ws():
+                pass
+
+
 @pytest.mark.skipif(msgpack, reason='test requires msgpack lib to be missing')
 def test_msgpack_missing():
-
     options = WebSocketOptions()
     handler = options.media_handlers[falcon.WebSocketPayloadType.BINARY]
 
@@ -1093,3 +1160,300 @@ def test_msgpack_missing():
 
     with pytest.raises(RuntimeError):
         handler.deserialize(b'{}')
+
+
+@pytest.mark.parametrize('reason', ['Client closing connection', '', None])
+async def test_client_close_with_reason(reason, conductor):
+    class Resource:
+        def __init__(self):
+            pass
+
+        async def on_websocket(self, req, ws):
+            await ws.accept()
+            while True:
+                try:
+                    await ws.receive_data()
+
+                except falcon.WebSocketDisconnected:
+                    break
+
+    resource = Resource()
+    conductor.app.add_route('/', resource)
+
+    async with conductor as c:
+        async with c.simulate_ws('/', spec_version='2.3') as ws:
+            await ws.close(4099, reason)
+
+    assert ws.close_code == 4099
+    if reason:
+        assert ws.close_reason == reason
+    else:
+        assert ws.close_reason == ''
+
+
+@pytest.mark.parametrize('reason', ['PEBCAK', 'wow such reason', '', None])
+async def test_close_with_reason_no_cm(conductor, reason):
+    class Resource:
+        async def on_websocket(self, req, ws):
+            await ws.accept()
+            text = await ws.receive_text()
+            await ws.send_text(text.upper())
+            await ws.close(4001, reason)
+
+    resource = Resource()
+    conductor.app.add_route('/', resource)
+
+    # NOTE(vytas): Here we don't use the async context manager pattern in order
+    #   to collect coverage under CPython 3.11. It doesn't seem to help though.
+    context = conductor.simulate_ws(spec_version='2.4')
+    ws = context._ws
+
+    await ws.wait_ready()
+    await ws.send_text('Hello, World!')
+    received = await ws.receive_text()
+    assert received == 'HELLO, WORLD!'
+
+    with pytest.raises(falcon.WebSocketDisconnected):
+        await ws.receive_text()
+
+    assert ws.close_reason == (reason or '')
+
+
+@pytest.mark.parametrize('reason', ['PEBCAK', 'wow such reason', '', None])
+@pytest.mark.parametrize('spec_version', ['2.2', '2.3', '2.4'])
+async def test_close_with_reason(conductor, reason, spec_version):
+    class Resource:
+        async def on_websocket(self, req, ws):
+            await ws.accept()
+            await ws.close(3400, reason)
+
+    resource = Resource()
+    conductor.app.add_route('/', resource)
+
+    async with conductor as c:
+        async with c.simulate_ws('/', spec_version=spec_version) as ws:
+            # Make sure the responder has a chance to reach the close() statement
+            for _ in range(3):
+                await asyncio.sleep(0)
+            assert ws.closed
+            assert ws.close_code == 3400
+
+    assert ws.close_code == 3400
+    if spec_version == '2.2':
+        assert ws.close_reason == ''
+    else:
+        assert ws.close_reason == reason or 'Bad Request'
+
+
+@pytest.mark.parametrize('no_default', [True, False])
+@pytest.mark.parametrize(
+    'code,expected',
+    [
+        (None, 'Normal Closure'),
+        (1011, 'Internal Server Error'),
+        (3405, 'Method Not Allowed'),
+        (3701, ''),
+        (3702, 'Emacs'),
+        (4042, ''),
+        (4099, 'wow such reason'),
+    ],
+)
+async def test_reason_mapping(no_default, code, expected, conductor):
+    class Resource:
+        def __init__(self):
+            pass
+
+        async def on_websocket(self, req, ws):
+            await ws.accept()
+            await ws.close(code)
+
+    resource = Resource()
+    conductor.app.add_route('/', resource)
+    if no_default:
+        conductor.app.ws_options.default_close_reasons = {}
+    else:
+        # NOTE(vytas): Although it would be fun, we opt not to provide reasons
+        #   for 7xx errors by default.
+        conductor.app.ws_options.default_close_reasons[3702] = 'Emacs'
+        conductor.app.ws_options.default_close_reasons[4099] = 'wow such reason'
+
+    async with conductor as c:
+        with pytest.raises(falcon.WebSocketDisconnected):
+            async with c.simulate_ws('/', spec_version='2.10.3') as ws:
+                await ws.receive_data()
+
+    if code:
+        assert ws.close_code == code
+    else:
+        assert ws.close_code == CloseCode.NORMAL
+
+    if no_default:
+        assert ws.close_reason == ''
+    else:
+        assert ws.close_reason == expected
+
+
+@pytest.mark.parametrize('status', [200, 500, 422, 400])
+@pytest.mark.parametrize('thing', [falcon.HTTPStatus, falcon.HTTPError])
+@pytest.mark.parametrize('accept', [True, False])
+async def test_ws_http_error_or_status_response(conductor, status, thing, accept):
+    class Resource:
+        async def on_websocket(self, req, ws):
+            if accept:
+                await ws.accept()
+            raise thing(status)
+
+    conductor.app.add_route('/', Resource())
+    exp_code = 3000 + status
+
+    async with conductor as c:
+        if accept:
+            async with c.simulate_ws() as ws:
+                # Make sure the responder has a chance to reach the raise point
+                for _ in range(3):
+                    await asyncio.sleep(0)
+                assert ws.closed
+                assert ws.close_code == exp_code
+        else:
+            with pytest.raises(falcon.WebSocketDisconnected) as err:
+                async with c.simulate_ws():
+                    pass
+            assert err.value.code == exp_code
+
+
+@pytest.mark.parametrize('status', [200, 500, 422, 400])
+@pytest.mark.parametrize(
+    'thing',
+    [
+        falcon.HTTPStatus,
+        falcon.HTTPError,
+    ],
+)
+@pytest.mark.parametrize('place', ['request', 'resource'])
+async def test_ws_http_error_or_status_middleware(conductor, status, thing, place):
+    called = False
+
+    class Resource:
+        async def on_websocket(self, req, ws):
+            nonlocal called
+            called = True
+
+    class Middleware:
+        async def process_request_ws(self, req, ws):
+            if place == 'request':
+                raise thing(status)
+
+        async def process_resource_ws(self, req, ws, res, params):
+            if place == 'resource':
+                raise thing(status)
+
+    conductor.app.add_route('/', Resource())
+    conductor.app.add_middleware(Middleware())
+    exp_code = 3000 + status
+
+    async with conductor as c:
+        with pytest.raises(falcon.WebSocketDisconnected) as err:
+            async with c.simulate_ws():
+                pass
+        assert err.value.code == exp_code
+    assert not called
+
+
+class FooBarError(Exception):
+    pass
+
+
+@pytest.mark.parametrize('status', [200, 500, 422, 400])
+@pytest.mark.parametrize('thing', [falcon.HTTPStatus, falcon.HTTPError])
+@pytest.mark.parametrize(
+    'place', ['request', 'resource', 'ws_before_accept', 'ws_after_accept']
+)
+@pytest.mark.parametrize('handler_has_ws', [True, False])
+async def test_ws_http_error_or_status_error_handler(
+    conductor, status, thing, place, handler_has_ws
+):
+    class Resource:
+        async def on_websocket(self, req, ws):
+            if place == 'ws_before_accept':
+                raise FooBarError
+            await ws.accept()
+            if place == 'ws_after_accept':
+                raise FooBarError
+
+    class Middleware:
+        async def process_request_ws(self, req, ws):
+            if place == 'request':
+                raise FooBarError
+
+        async def process_resource_ws(self, req, ws, res, params):
+            if place == 'resource':
+                raise FooBarError
+
+    if handler_has_ws:
+
+        async def handle_foobar(req, resp, ex, param, ws=None):
+            raise thing(status)
+
+    else:
+
+        async def handle_foobar(req, resp, ex, param):  # type: ignore[misc]
+            raise thing(status)
+
+    conductor.app.add_route('/', Resource())
+    conductor.app.add_middleware(Middleware())
+    conductor.app.add_error_handler(FooBarError, handle_foobar)
+    exp_code = 3000 + status
+
+    async with conductor as c:
+        if place == 'ws_after_accept':
+            async with c.simulate_ws() as ws:
+                # Make sure the responder has a chance to reach the raise point
+                for _ in range(3):
+                    await asyncio.sleep(0)
+                assert ws.closed
+                assert ws.close_code == exp_code
+        else:
+            with pytest.raises(falcon.WebSocketDisconnected) as err:
+                async with c.simulate_ws():
+                    pass
+            assert err.value.code == exp_code
+
+
+@pytest.mark.parametrize('max_receive_queue', [0, 1, 4, 7, 17])
+async def test_max_receive_queue_sizes(conductor, max_receive_queue):
+    class Chat:
+        async def on_websocket(self, req, ws, user):
+            await ws.accept()
+
+            broadcast = [
+                '[Guest123] ping',
+                '[John F. Barbaz] Hi everyone.',
+                f'Hello, {user}!',
+            ]
+
+            while True:
+                await ws.send_text(broadcast.pop())
+
+                msg = await ws.receive_text()
+                if msg == '/quit':
+                    await ws.send_text('Bye!')
+                    await ws.close(reason='Received /quit')
+                    break
+                else:
+                    await ws.send_text(f'[{user}] {msg}')
+
+    conductor.app.ws_options.max_receive_queue = max_receive_queue
+    conductor.app.add_route('/chat/{user}', Chat())
+
+    received = []
+    messages = ['/quit', 'I have to leave this test soon.', 'Hello!']
+
+    async with conductor as c:
+        async with c.simulate_ws('/chat/foobarer') as ws:
+            while messages:
+                received.append(await ws.receive_text())
+
+                await ws.send_text(messages.pop())
+                received.append(await ws.receive_text())
+
+    assert len(received) == 6
