@@ -23,5 +23,4 @@ async def app(scope, receive, send):
         }
     )
 
-    loop = asyncio.get_event_loop()
-    loop.create_task(_say_hi())
+    asyncio.create_task(_say_hi())
