@@ -15,7 +15,7 @@ class CookieResource:
 
 @pytest.fixture
 def alias_client():
-    with pytest.warns(DeprecatedWarning, match='API class may be removed'):
+    with pytest.warns(DeprecatedWarning, match='API class will be removed'):
         api = falcon.API()
     api.add_route('/get-cookie', CookieResource())
     return testing.TestClient(api)
@@ -42,6 +42,6 @@ def test_cookies(alias_client, app_client):
 
 
 def test_alias_equals_to_app(alias_client):
-    with pytest.warns(DeprecatedWarning, match='API class may be removed'):
+    with pytest.warns(DeprecatedWarning, match='API class will be removed'):
         api = falcon.API()
     assert isinstance(api, falcon.API)
