@@ -4,14 +4,18 @@ import sys
 
 import pytest
 
-from falcon import App, inspect
+from falcon import App
+from falcon import inspect
+import falcon.asgi
 from falcon.cmd import inspect_app
 from falcon.testing import redirected
 
-from _util import create_app  # NOQA
-
 _WIN32 = sys.platform.startswith('win')
-_MODULE = 'tests.test_cmd_inspect_app'
+
+# NOTE(vytas): This is not the cleanest way to import as we lack __init__.py,
+#   but it works as pytest (when operating in the default "prepend" import mode)
+#   inserts the directory of every test file into sys.path.
+_MODULE = 'test_cmd_inspect_app'
 
 
 class DummyResource:
@@ -26,6 +30,11 @@ class DummyResourceAsync:
         resp.status = '200 OK'
 
 
+def create_app(asgi):
+    app_cls = falcon.asgi.App if asgi else App
+    return app_cls()
+
+
 def make_app(asgi=False):
     app = create_app(asgi)
     app.add_route('/test', DummyResourceAsync() if asgi else DummyResource())
@@ -189,8 +198,9 @@ def test_route_main(monkeypatch):
 
     monkeypatch.setattr(inspect_app, 'main', mock)
     output = io.StringIO()
-    with redirected(stdout=output):
-        inspect_app.route_main()
+    with redirected(stderr=output):
+        with pytest.raises(SystemExit):
+            inspect_app.route_main()
 
-    assert 'deprecated' in output.getvalue()
-    assert called
+    assert 'no longer supported' in output.getvalue()
+    assert not called
