@@ -1,23 +1,23 @@
+from pathlib import Path
+
 import pytest
 
 import falcon
 from falcon import testing
 
-from _util import create_app, disable_asgi_non_coroutine_wrapping  # NOQA
-
 
 @pytest.fixture
-def client(asgi):
-    app = create_app(asgi)
+def client(asgi, util):
+    app = util.create_app(asgi)
     return testing.TestClient(app)
 
 
 @pytest.fixture(scope='function')
-def cors_client(asgi):
+def cors_client(asgi, util):
     # NOTE(kgriffs): Disable wrapping to test that built-in middleware does
     #   not require it (since this will be the case for non-test apps).
-    with disable_asgi_non_coroutine_wrapping():
-        app = create_app(asgi, cors_enable=True)
+    with util.disable_asgi_non_coroutine_wrapping():
+        app = util.create_app(asgi, cors_enable=True)
     return testing.TestClient(app)
 
 
@@ -30,6 +30,12 @@ class CORSHeaderResource:
         resp.text = "I'm a CORS test response"
 
 
+class CORSOptionsResource:
+    def on_options(self, req, resp):
+        # No allow header set
+        resp.set_header('Content-Length', '0')
+
+
 class TestCorsMiddleware:
     def test_disabled_cors_should_not_add_any_extra_headers(self, client):
         client.app.add_route('/', CORSHeaderResource())
@@ -82,6 +88,21 @@ class TestCorsMiddleware:
             result.headers['Access-Control-Max-Age'] == '86400'
         )  # 24 hours in seconds
 
+    def test_enabled_cors_handles_preflight_custom_option(self, cors_client):
+        cors_client.app.add_route('/', CORSOptionsResource())
+        result = cors_client.simulate_options(
+            headers=(
+                ('Origin', 'localhost'),
+                ('Access-Control-Request-Method', 'GET'),
+                ('Access-Control-Request-Headers', 'X-PINGOTHER, Content-Type'),
+            )
+        )
+        assert 'Access-Control-Allow-Methods' not in result.headers
+        assert 'Access-Control-Allow-Headers' not in result.headers
+        assert 'Access-Control-Max-Age' not in result.headers
+        assert 'Access-Control-Expose-Headers' not in result.headers
+        assert 'Access-Control-Allow-Origin' not in result.headers
+
     def test_enabled_cors_handles_preflighting_no_headers_in_req(self, cors_client):
         cors_client.app.add_route('/', CORSHeaderResource())
         result = cors_client.simulate_options(
@@ -96,11 +117,55 @@ class TestCorsMiddleware:
             result.headers['Access-Control-Max-Age'] == '86400'
         )  # 24 hours in seconds
 
+    def test_enabled_cors_static_route(self, cors_client):
+        cors_client.app.add_static_route('/static', Path(__file__).parent)
+        result = cors_client.simulate_options(
+            f'/static/{Path(__file__).name}',
+            headers=(
+                ('Origin', 'localhost'),
+                ('Access-Control-Request-Method', 'GET'),
+            ),
+        )
+
+        assert result.headers['Access-Control-Allow-Methods'] == 'GET'
+        assert result.headers['Access-Control-Allow-Headers'] == '*'
+        assert result.headers['Access-Control-Max-Age'] == '86400'
+        assert result.headers['Access-Control-Allow-Origin'] == '*'
+
+    @pytest.mark.parametrize('support_options', [True, False])
+    def test_enabled_cors_sink_route(self, cors_client, support_options):
+        def my_sink(req, resp):
+            if req.method == 'OPTIONS' and support_options:
+                resp.set_header('ALLOW', 'GET')
+            else:
+                resp.text = 'my sink'
+
+        cors_client.app.add_sink(my_sink, '/sink')
+        result = cors_client.simulate_options(
+            '/sink/123',
+            headers=(
+                ('Origin', 'localhost'),
+                ('Access-Control-Request-Method', 'GET'),
+            ),
+        )
+
+        if support_options:
+            assert result.headers['Access-Control-Allow-Methods'] == 'GET'
+            assert result.headers['Access-Control-Allow-Headers'] == '*'
+            assert result.headers['Access-Control-Max-Age'] == '86400'
+            assert result.headers['Access-Control-Allow-Origin'] == '*'
+        else:
+            assert 'Access-Control-Allow-Methods' not in result.headers
+            assert 'Access-Control-Allow-Headers' not in result.headers
+            assert 'Access-Control-Max-Age' not in result.headers
+            assert 'Access-Control-Expose-Headers' not in result.headers
+            assert 'Access-Control-Allow-Origin' not in result.headers
+
 
 @pytest.fixture(scope='function')
-def make_cors_client(asgi):
+def make_cors_client(asgi, util):
     def make(middleware):
-        app = create_app(asgi, middleware=middleware)
+        app = util.create_app(asgi, middleware=middleware)
         return testing.TestClient(app)
 
     return make
