@@ -5,11 +5,9 @@ import pytest
 from falcon import testing
 from falcon.routing import DefaultRouter
 
-from _util import create_app  # NOQA
 
-
-def client(asgi):
-    return testing.TestClient(create_app(asgi))
+def client(asgi, util):
+    return testing.TestClient(util.create_app(asgi))
 
 
 @pytest.fixture
@@ -250,10 +248,9 @@ def test_user_regression_special_chars(uri_template, path, expected_params):
 # =====================================================================
 
 
-@pytest.mark.parametrize('asgi', [True, False])
 @pytest.mark.parametrize('uri_template', [{}, set(), object()])
-def test_not_str(asgi, uri_template):
-    app = create_app(asgi)
+def test_not_str(asgi, util, uri_template):
+    app = util.create_app(asgi)
     with pytest.raises(TypeError):
         app.add_route(uri_template, ResourceWithId(-1))
 
@@ -727,3 +724,96 @@ def test_params_in_non_taken_branches(param_router, route, expected, num):
 
     assert resource.resource_id == num
     assert params == expected
+
+
+# capture path
+
+
+def test_capture_path_no_children():
+    router = DefaultRouter()
+    router.add_route('/foo/{bar:path}', ResourceWithId(1))
+    res = router.finder_src
+    with pytest.raises(
+        ValueError,
+        match='Cannot add route with template "/foo/{bar:path}/child". '
+        'Field name "bar" uses the converter "path"',
+    ):
+        router.add_route('/foo/{bar:path}/child', ResourceWithId(1))
+    with pytest.raises(
+        ValueError,
+        match='Cannot add route with template "/{bar:path}/child". '
+        'Field name "bar" uses the converter "path"',
+    ):
+        router.add_route('/{bar:path}/child', ResourceWithId(1))
+    assert res == router.finder_src
+
+
+@pytest.mark.parametrize(
+    'template',
+    (
+        '/foo/{bar:path}-x',
+        '/foo/x-{bar:path}',
+        '/foo/{x}-{bar:path}',
+        '/foo/{bar:path}-{x}',
+    ),
+)
+def test_capture_path_complex(template):
+    router = DefaultRouter()
+    with pytest.raises(
+        ValueError, match='Cannot use converter "path" of variable "bar" in a template '
+    ):
+        router.add_route(template, ResourceWithId(1))
+
+
+@pytest.fixture
+def capture_path_router():
+    router = DefaultRouter()
+
+    router.add_route('/foo/bar/baz', ResourceWithId(1))
+    router.add_route('/foo/{bar:path}', ResourceWithId(2))
+    router.add_route('/foo/bar/{foo:path}', ResourceWithId(3))
+    router.add_route('/{baz:path}', ResourceWithId(4))
+    router.add_route('/x/{v1:int}/{v2}/{other:path}', ResourceWithId(5))
+    router.add_route('/y/{v1:int}/{v2:int}/{other:path}', ResourceWithId(6))
+    return router
+
+
+@pytest.mark.parametrize(
+    'route, expected, num',
+    (
+        ('/foo/bar/baz', {}, 1),
+        ('/foo/some/path/here', {'bar': 'some/path/here'}, 2),
+        ('/foo/bar/bar', {'foo': 'bar'}, 3),
+        (
+            '/foo/bar/bar-1/2/3/4/5/5/6/7/8/98/9/0/-/9/',
+            {'foo': 'bar-1/2/3/4/5/5/6/7/8/98/9/0/-/9/'},
+            3,
+        ),
+        ('/x/1/2/3', {'v1': 1, 'v2': '2', 'other': '3'}, 5),
+        ('/x/1/2/3/4/5/6', {'v1': 1, 'v2': '2', 'other': '3/4/5/6'}, 5),
+        ('/upload/youtube/auth/token', {'baz': 'upload/youtube/auth/token'}, 4),
+        ('/x/y/o.o/w', {'baz': 'x/y/o.o/w'}, 4),
+        ('/foo', {'baz': 'foo'}, 4),
+        ('/foo/', {'bar': ''}, 2),
+        ('/foo/bar', {'bar': 'bar'}, 2),
+        ('/foo/bar/', {'foo': ''}, 3),
+        ('/foo/bar/baz/other', {'foo': 'baz/other'}, 3),
+        ('/y/1/2/3', {'v1': 1, 'v2': 2, 'other': '3'}, 6),
+        ('/y/1/a/3', {'baz': 'y/1/a/3'}, 4),
+    ),
+)
+def test_capture_path(capture_path_router, route, expected, num):
+    resource, __, params, __ = capture_path_router.find(route)
+
+    assert resource.resource_id == num
+    assert params == expected
+
+
+def test_capture_path_no_match():
+    router = DefaultRouter()
+
+    router.add_route('/foo/bar/baz', ResourceWithId(1))
+    router.add_route('/foo/{bar:path}', ResourceWithId(2))
+    router.add_route('/foo/bar/{foo:path}', ResourceWithId(3))
+
+    assert router.find('/foo') is None
