@@ -9,9 +9,6 @@ from falcon import testing
 import falcon.constants
 from falcon.routing import util
 
-from _util import create_app, has_cython  # NOQA
-
-
 FALCON_CUSTOM_HTTP_METHODS = ['FOO', 'BAR']
 
 
@@ -34,10 +31,10 @@ def cleanup_constants():
 
 
 @pytest.fixture
-def custom_http_client(asgi, request, cleanup_constants, resource_things):
+def custom_http_client(asgi, util, request, cleanup_constants, resource_things):
     falcon.constants.COMBINED_METHODS += FALCON_CUSTOM_HTTP_METHODS
 
-    app = create_app(asgi)
+    app = util.create_app(asgi)
     app.add_route('/things', resource_things)
     return testing.TestClient(app)
 
@@ -63,7 +60,6 @@ def test_map_http_methods(custom_http_client, resource_things):
     assert 'BAR' not in method_map
 
 
-@pytest.mark.skipif(has_cython, reason='Reloading modules on Cython does not work')
 @pytest.mark.parametrize(
     'env_str,expected',
     [
@@ -75,7 +71,12 @@ def test_map_http_methods(custom_http_client, resource_things):
         (' foo , BAR ', ['FOO', 'BAR']),
     ],
 )
-def test_environment_override(cleanup_constants, resource_things, env_str, expected):
+def test_environment_override(
+    util, cleanup_constants, resource_things, env_str, expected
+):
+    if util.HAS_CYTHON:
+        pytest.skip(reason='Reloading modules on Cython does not work')
+
     # Make sure we don't have anything in there
     for method in expected:
         assert method not in falcon.constants.COMBINED_METHODS
@@ -105,6 +106,7 @@ def test_foo(custom_http_client, resource_things):
 
     assert 'FOO' in falcon.constants.COMBINED_METHODS
     assert response.status == falcon.HTTP_204
+    assert response.status_code == 204
     assert resource_things.called
 
 
@@ -123,3 +125,4 @@ def test_bar(custom_http_client, resource_things):
 
     assert 'BAR' in falcon.constants.COMBINED_METHODS
     assert response.status == falcon.HTTP_405
+    assert response.status_code == 405
