@@ -1,24 +1,28 @@
-# -*- coding: utf-8
-
 import datetime
+import http
 import json
 import wsgiref.validate
 import xml.etree.ElementTree as et  # noqa: I202
 
 import pytest
-import yaml
 
 import falcon
-from falcon.http_error import NoRepresentation, OptionalRepresentation
+from falcon.constants import MEDIA_JSON
+from falcon.constants import MEDIA_XML
+from falcon.constants import MEDIA_YAML
+from falcon.media import BaseHandler
 import falcon.testing as testing
 from falcon.util.deprecation import DeprecatedWarning
 
-from _util import create_app  # NOQA
+try:
+    import yaml
+except ImportError:
+    yaml = None  # type: ignore[assignment]
 
 
 @pytest.fixture
-def client(asgi):
-    app = create_app(asgi)
+def client(asgi, util):
+    app = util.create_app(asgi)
 
     resource = FaultyResource()
     app.add_route('/fail', resource)
@@ -26,6 +30,22 @@ def client(asgi):
     return testing.TestClient(app)
 
 
+@pytest.fixture(
+    params=[
+        pytest.param(True, id='with_xml'),
+        pytest.param(False, id='without_xml'),
+        pytest.param(None, id='default_xml'),
+    ]
+)
+def enable_xml(request):
+    def go(app):
+        if request.param is not None:
+            app.resp_options.xml_error_serialization = request.param
+        return request.param is not False
+
+    return go
+
+
 class FaultyResource:
     def on_get(self, req, resp):
         status = req.get_header('X-Error-Status')
@@ -33,6 +53,21 @@ class FaultyResource:
         description = req.get_header('X-Error-Description')
         code = 10042
 
+        status_type = req.get_header('X-Error-Status-Type')
+        if status_type:
+            assert status
+
+        if status_type == 'int':
+            status = int(status)
+        elif status_type == 'bytes':
+            status = status.encode()
+        elif status_type == 'HTTPStatus':
+            status = http.HTTPStatus(int(status))
+        elif status_type == 'str':
+            pass
+        elif status_type is not None:
+            pytest.fail(f'status_type {status_type} not recognized')
+
         raise falcon.HTTPError(status, title=title, description=description, code=code)
 
     def on_post(self, req, resp):
@@ -63,7 +98,7 @@ class UnicodeFaultyResource:
     def on_get(self, req, resp):
         self.called = True
         raise falcon.HTTPError(
-            falcon.HTTP_792,
+            792,  # NOTE(kgriffs): Test that an int is acceptable even for 7xx codes
             title='Internet \xe7rashed!',
             description='\xc7atastrophic weather event',
             href='http://example.com/api/\xe7limate',
@@ -157,7 +192,7 @@ class LengthRequiredResource:
 
 class RequestEntityTooLongResource:
     def on_get(self, req, resp):
-        raise falcon.HTTPPayloadTooLarge(
+        raise falcon.HTTPContentTooLarge(
             title='Request Rejected', description='Request Body Too Large'
         )
 
@@ -167,7 +202,7 @@ class TemporaryRequestEntityTooLongResource:
         self.retry_after = retry_after
 
     def on_get(self, req, resp):
-        raise falcon.HTTPPayloadTooLarge(
+        raise falcon.HTTPContentTooLarge(
             title='Request Rejected',
             description='Request Body Too Large',
             retry_after=self.retry_after,
@@ -280,17 +315,14 @@ class TestHTTPError:
         assert response.status == headers['X-Error-Status']
         assert response.json == expected_body
 
-    def test_has_representation(self):
-        with pytest.warns(DeprecatedWarning, match='has_representation is deprecated'):
-            assert falcon.HTTPError(falcon.HTTP_701).has_representation is True
-
     def test_no_description_json(self, client):
         response = client.simulate_patch('/fail')
         assert response.status == falcon.HTTP_400
         assert response.json == {'title': '400 Bad Request'}
-        assert response.headers['Content-Type'] == 'application/json'
+        assert response.content_type == 'application/json'
 
     def test_no_description_xml(self, client):
+        client.app.resp_options.xml_error_serialization = True
         response = client.simulate_patch(
             path='/fail', headers={'Accept': 'application/xml'}
         )
@@ -302,7 +334,36 @@ class TestHTTPError:
         )
 
         assert response.content == expected_xml
-        assert response.headers['Content-Type'] == 'application/xml'
+        assert response.content_type == 'application/xml'
+
+    @pytest.mark.parametrize('custom_xml', [True, False])
+    def test_xml_enable(self, client, enable_xml, custom_xml):
+        has_xml = enable_xml(client.app)
+        client.app.resp_options.default_media_type = 'app/foo'
+        accept = 'app/falcon+xml' if custom_xml else 'application/xml'
+        response = client.simulate_patch(path='/fail', headers={'Accept': accept})
+        assert response.status == falcon.HTTP_400
+
+        if has_xml:
+            expected_xml = (
+                b'<?xml version="1.0" encoding="UTF-8"?><error>'
+                b'<title>400 Bad Request</title></error>'
+            )
+            assert response.content == expected_xml
+        else:
+            assert response.content == b''
+        if has_xml or custom_xml:
+            assert response.content_type == 'application/xml'
+        else:
+            assert response.content_type == 'app/foo'
+
+    def test_to_xml_deprecated(self):
+        with pytest.warns(
+            DeprecatedWarning,
+            match='The internal error serialization to XML is deprecated.',
+        ):
+            res = falcon.HTTPGone().to_xml()
+        assert res == falcon.HTTPGone()._to_xml()
 
     def test_client_does_not_accept_json_or_xml(self, client):
         headers = {
@@ -321,6 +382,7 @@ class TestHTTPError:
         assert response.headers['Vary'] == 'Accept'
         assert not response.content
 
+    @pytest.mark.skipif(yaml is None, reason='PyYAML is required for this test')
     def test_custom_error_serializer(self, client):
         headers = {
             'X-Error-Title': 'Storage service down',
@@ -484,6 +546,7 @@ class TestHTTPError:
         ],
     )
     def test_epic_fail_xml(self, client, media_type):
+        client.app.resp_options.xml_error_serialization = True
         headers = {'Accept': media_type}
 
         expected_body = (
@@ -532,6 +595,7 @@ class TestHTTPError:
         assert expected_body == response.json
 
     def test_unicode_xml(self, client):
+        client.app.resp_options.xml_error_serialization = True
         unicode_resource = UnicodeFaultyResource()
 
         expected_body = (
@@ -721,13 +785,14 @@ class TestHTTPError:
         parsed_body = json.loads(response.content.decode())
         assert parsed_body['code'] == code
 
-    def test_416(self, client, asgi):
-        client.app = create_app(asgi)
+    def test_416(self, client, asgi, util):
+        client.app = util.create_app(asgi)
+        client.app.resp_options.xml_error_serialization = True
         client.app.add_route('/416', RangeNotSatisfiableResource())
         response = client.simulate_request(path='/416', headers={'accept': 'text/xml'})
 
         assert response.status == falcon.HTTP_416
-        assert response.content == falcon.HTTPRangeNotSatisfiable(123456).to_xml()
+        assert response.content == falcon.HTTPRangeNotSatisfiable(123456)._to_xml()
         exp = (
             b'<?xml version="1.0" encoding="UTF-8"?><error>'
             b'<title>416 Range Not Satisfiable</title></error>'
@@ -873,16 +938,30 @@ class TestHTTPError:
         self._misc_test(client, falcon.HTTPInternalServerError, falcon.HTTP_500)
         self._misc_test(client, falcon.HTTPBadGateway, falcon.HTTP_502)
 
-    def test_title_default_message_if_none(self, client):
-        headers = {'X-Error-Status': falcon.HTTP_503}
+    @pytest.mark.parametrize(
+        'status, status_type',
+        [
+            (falcon.HTTP_503, 'str'),
+            (falcon.HTTP_503, 'bytes'),
+            (503, 'int'),
+            (503, 'str'),
+            (503, 'bytes'),
+            (503, 'HTTPStatus'),
+        ],
+    )
+    def test_title_default_message_if_none(self, status, status_type, client):
+        headers = {
+            'X-Error-Status': str(status),
+            'X-Error-Status-Type': status_type,
+        }
 
         response = client.simulate_request(path='/fail', headers=headers)
 
-        assert response.status == headers['X-Error-Status']
-        assert response.json['title'] == headers['X-Error-Status']
+        assert response.json['title'] == falcon.HTTP_503
+        assert response.status_code == 503
 
     def test_to_json_dumps(self):
-        e = falcon.HTTPError(status=falcon.HTTP_418, title='foo', description='bar')
+        e = falcon.HTTPError(status=418, title='foo', description='bar')
         assert e.to_json() == b'{"title": "foo", "description": "bar"}'
 
         class Handler:
@@ -918,38 +997,141 @@ class TestHTTPError:
         err.__cause__ = ValueError('some error')
         assert err.description == 'Could not parse foo-media body - some error'
 
-
-def test_kw_only():
-    # only deprecated for now
-    # with pytest.raises(TypeError, match='positional argument'):
-    #     falcon.HTTPError(falcon.HTTP_BAD_REQUEST, 'foo', 'bar')
-    with pytest.warns(DeprecatedWarning, match='positional args are deprecated'):
-        falcon.HTTPError(falcon.HTTP_BAD_REQUEST, 'foo', 'bar')
+    def test_kw_only(self):
+        with pytest.raises(TypeError, match='positional argument'):
+            falcon.HTTPError(falcon.HTTP_BAD_REQUEST, 'foo', 'bar')
 
 
-def test_NoRepresentation():
-    with pytest.warns(
-        DeprecatedWarning,
-        match='has_representation is deprecated.*The class NoRepresentation',
+JSON_CONTENT = b'{"title": "410 Gone"}'
+JSON = (MEDIA_JSON, MEDIA_JSON, JSON_CONTENT)
+CUSTOM_JSON = ('custom/any+json', MEDIA_JSON, JSON_CONTENT)
+
+XML_CONTENT = (
+    b'<?xml version="1.0" encoding="UTF-8"?><error><title>410 Gone</title></error>'
+)
+XML = (MEDIA_XML, MEDIA_XML, XML_CONTENT)
+CUSTOM_XML = ('custom/any+xml', MEDIA_XML, XML_CONTENT)
+
+YAML = (MEDIA_YAML, MEDIA_YAML, b'title: 410 Gone!')
+ASYNC_ONLY = ('application/only_async', 'application/only_async', b'this is async')
+ASYNC_WITH_SYNC = (
+    'application/async_with_sync',
+    'application/async_with_sync',
+    b'this is sync instead',
+)
+
+
+class FakeYamlMediaHandler(BaseHandler):
+    def serialize(self, media, content_type=None):
+        assert media == {'title': '410 Gone'}
+        return b'title: 410 Gone!'
+
+
+class AsyncOnlyMediaHandler(BaseHandler):
+    async def serialize_async(self, media, content_type=None):
+        assert media == {'title': '410 Gone'}
+        return b'this is async'
+
+
+class SyncInterfaceMediaHandler(AsyncOnlyMediaHandler):
+    def serialize(self, media, content_type=None):
+        assert media == {'title': '410 Gone'}
+        return b'this is sync instead'
+
+    _serialize_sync = serialize  # type: ignore[assignment]
+
+
+class TestDefaultSerializeError:
+    @pytest.fixture
+    def client(self, util, asgi):
+        app = util.create_app(asgi)
+        app.add_route('/', GoneResource())
+        return testing.TestClient(app)
+
+    def test_unknown_accept(self, client):
+        res = client.simulate_get(headers={'Accept': 'foo/bar'})
+        assert res.content_type == 'application/json'
+        assert res.headers['vary'] == 'Accept'
+        assert res.content == b''
+
+    @pytest.mark.parametrize('has_json_handler', [True, False])
+    def test_defaults_to_json(self, client, has_json_handler):
+        if not has_json_handler:
+            client.app.req_options.media_handlers.pop(MEDIA_JSON)
+            client.app.resp_options.media_handlers.pop(MEDIA_JSON)
+        res = client.simulate_get()
+        assert res.content_type == 'application/json'
+        assert res.headers['vary'] == 'Accept'
+        assert res.content == JSON_CONTENT
+
+    @pytest.mark.parametrize(
+        'accept, content_type, content',
+        (JSON, XML, CUSTOM_JSON, CUSTOM_XML, YAML, ASYNC_ONLY, ASYNC_WITH_SYNC),
+    )
+    def test_serializes_error_to_preferred_by_sender(
+        self, accept, content_type, content, client, asgi
     ):
-        assert NoRepresentation().has_representation is False
+        client.app.resp_options.xml_error_serialization = True
+        client.app.resp_options.media_handlers[MEDIA_YAML] = FakeYamlMediaHandler()
+        client.app.resp_options.media_handlers[ASYNC_WITH_SYNC[0]] = (
+            SyncInterfaceMediaHandler()
+        )
+        if asgi:
+            client.app.resp_options.media_handlers[ASYNC_ONLY[0]] = (
+                AsyncOnlyMediaHandler()
+            )
+        res = client.simulate_get(headers={'Accept': accept})
+        assert res.headers['vary'] == 'Accept'
+        if content_type == ASYNC_ONLY[0] and not asgi:
+            # media-json is the default content type
+            assert res.content_type == MEDIA_JSON
+            assert res.content == b''
+        else:
+            assert res.content_type == content_type
+            assert res.content == content
 
+    def test_json_async_only_error(self, util):
+        app = util.create_app(True)
+        app.add_route('/', GoneResource())
+        app.resp_options.media_handlers[MEDIA_JSON] = AsyncOnlyMediaHandler()
+        client = testing.TestClient(app)
+        with pytest.raises(NotImplementedError, match='requires the sync interface'):
+            client.simulate_get()
 
-class TestOptionalRepresentation:
-    def test_OptionalRepresentation_false(self):
-        with pytest.warns(
-            DeprecatedWarning,
-            match='has_representation is deprecated.*The class OptionalRepresentation',
-        ):
-            or_ = OptionalRepresentation()
-            or_.description = None
-            assert or_.has_representation is False
+    @pytest.mark.parametrize('accept', [MEDIA_XML, 'application/xhtml+xml'])
+    def test_add_xml_handler(self, client, enable_xml, accept):
+        enable_xml(client.app)
+        client.app.resp_options.media_handlers[MEDIA_XML] = FakeYamlMediaHandler()
+        res = client.simulate_get(headers={'Accept': accept})
+        assert res.content_type == MEDIA_XML
+        assert res.content == YAML[-1]
 
-    def test_OptionalRepresentation_true(self):
-        with pytest.warns(
-            DeprecatedWarning,
-            match='has_representation is deprecated.*The class OptionalRepresentation',
-        ):
-            or_ = OptionalRepresentation()
-            or_.description = 'foo'
-            assert or_.has_representation is True
+    @pytest.mark.parametrize(
+        'accept, content_type',
+        [
+            (
+                # firefox
+                'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,'
+                'image/webp,image/png,image/svg+xml,*/*;q=0.8',
+                MEDIA_XML,
+            ),
+            (
+                # safari / chrome
+                'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,'
+                'image/apng,*/*;q=0.8',
+                MEDIA_XML,
+            ),
+            ('text/html, application/xhtml+xml, image/jxr, */*', MEDIA_JSON),  # edge
+            (f'text/html,{MEDIA_YAML};q=0.8,*/*;q=0.7', MEDIA_YAML),
+            (f'text/html,{MEDIA_YAML};q=0.8,{MEDIA_JSON};q=0.8', MEDIA_JSON),
+        ],
+    )
+    def test_hard_content_types(self, client, accept, content_type, enable_xml):
+        has_xml = enable_xml(client.app)
+        client.app.resp_options.default_media_type = 'my_type'
+        client.app.resp_options.media_handlers[MEDIA_YAML] = FakeYamlMediaHandler()
+        res = client.simulate_get(headers={'Accept': accept})
+        if has_xml or content_type != MEDIA_XML:
+            assert res.content_type == content_type
+        else:
+            assert res.content_type == MEDIA_JSON
