@@ -1,5 +1,3 @@
-# -*- coding: utf-8
-
 import http
 
 import pytest
@@ -7,21 +5,18 @@ import pytest
 import falcon
 from falcon.http_status import HTTPStatus
 import falcon.testing as testing
-from falcon.util.deprecation import DeprecatedWarning
-
-from _util import create_app  # NOQA
 
 
-@pytest.fixture(params=[True, False])
-def client(request):
-    app = create_app(asgi=request.param)
+@pytest.fixture()
+def client(asgi, util):
+    app = util.create_app(asgi)
     app.add_route('/status', TestStatusResource())
     return testing.TestClient(app)
 
 
-@pytest.fixture(params=[True, False])
-def hook_test_client(request):
-    app = create_app(asgi=request.param)
+@pytest.fixture()
+def hook_test_client(asgi, util):
+    app = util.create_app(asgi)
     app.add_route('/status', TestHookResource())
     return testing.TestClient(app)
 
@@ -67,20 +62,17 @@ class TestStatusResource:
 
     @falcon.after(noop_after_hook)
     def on_delete(self, req, resp):
-        raise HTTPStatus(falcon.HTTP_200, headers={'X-Failed': 'False'}, text='Pass')
+        raise HTTPStatus(201, headers={'X-Failed': 'False'}, text='Pass')
 
 
 class TestHookResource:
     def on_get(self, req, resp):
-        resp.status = falcon.HTTP_500
+        resp.status_code = 500
         resp.set_header('X-Failed', 'True')
         resp.text = 'Fail'
 
     def on_patch(self, req, resp):
-        raise HTTPStatus(falcon.HTTP_200, text=None)
-
-    def on_delete(self, req, resp):
-        raise HTTPStatus(falcon.HTTP_200, headers={'X-Failed': 'False'}, text='Pass')
+        raise HTTPStatus(200, text=None)
 
 
 class TestHTTPStatus:
@@ -88,6 +80,7 @@ class TestHTTPStatus:
         """Make sure we get the 200 raised by before hook"""
         response = client.simulate_request(path='/status', method='GET')
         assert response.status == falcon.HTTP_200
+        assert response.status_code == 200
         assert response.headers['x-failed'] == 'False'
         assert response.text == 'Pass'
 
@@ -95,6 +88,7 @@ class TestHTTPStatus:
         """Make sure we get the 200 raised by responder"""
         response = client.simulate_request(path='/status', method='POST')
         assert response.status == falcon.HTTP_200
+        assert response.status_code == 200
         assert response.headers['x-failed'] == 'False'
         assert response.text == 'Pass'
 
@@ -102,13 +96,15 @@ class TestHTTPStatus:
         """Make sure after hooks still run"""
         response = client.simulate_request(path='/status', method='PUT')
         assert response.status == falcon.HTTP_200
+        assert response.status_code == 200
         assert response.headers['x-failed'] == 'False'
         assert response.text == 'Pass'
 
     def test_raise_status_survives_after_hooks(self, client):
         """Make sure after hook doesn't overwrite our status"""
         response = client.simulate_request(path='/status', method='DELETE')
-        assert response.status == falcon.HTTP_200
+        assert response.status == falcon.HTTP_201
+        assert response.status_code == 201
         assert response.headers['x-failed'] == 'False'
         assert response.text == 'Pass'
 
@@ -137,7 +133,7 @@ class TestHTTPStatusWithMiddleware:
         client.app.add_middleware(TestMiddleware())
 
         response = client.simulate_request(path='/status', method='GET')
-        assert response.status == falcon.HTTP_200
+        assert response.status_code == 200
         assert response.headers['x-failed'] == 'False'
         assert response.text == 'Pass'
 
@@ -188,11 +184,15 @@ class TestHTTPStatusWithMiddleware:
 class NoBodyResource:
     def on_get(self, req, res):
         res.data = b'foo'
-        raise HTTPStatus(falcon.HTTP_745)
+        http_status = HTTPStatus(745)
+        assert http_status.status_code == 745
+        raise http_status
 
     def on_post(self, req, res):
         res.media = {'a': 1}
-        raise HTTPStatus(falcon.HTTP_725)
+        http_status = HTTPStatus(falcon.HTTP_725)
+        assert http_status.status_code == 725
+        raise http_status
 
     def on_put(self, req, res):
         res.text = 'foo'
@@ -200,8 +200,8 @@ class NoBodyResource:
 
 
 @pytest.fixture()
-def body_client(asgi):
-    app = create_app(asgi=asgi)
+def body_client(asgi, util):
+    app = util.create_app(asgi)
     app.add_route('/status', NoBodyResource())
     return testing.TestClient(app)
 
@@ -210,21 +210,24 @@ class TestNoBodyWithStatus:
     def test_data_is_set(self, body_client):
         res = body_client.simulate_get('/status')
         assert res.status == falcon.HTTP_745
+        assert res.status_code == 745
         assert res.content == b''
 
     def test_media_is_set(self, body_client):
         res = body_client.simulate_post('/status')
         assert res.status == falcon.HTTP_725
+        assert res.status_code == 725
         assert res.content == b''
 
     def test_body_is_set(self, body_client):
         res = body_client.simulate_put('/status')
         assert res.status == falcon.HTTP_719
+        assert res.status_code == 719
         assert res.content == b''
 
 
 @pytest.fixture()
-def custom_status_client(asgi):
+def custom_status_client(asgi, util):
     def client(status):
         class Resource:
             def on_get(self, req, resp):
@@ -232,7 +235,7 @@ def custom_status_client(asgi):
                 resp.data = b'Hello, World!'
                 resp.status = status
 
-        app = create_app(asgi=asgi)
+        app = util.create_app(asgi)
         app.add_route('/status', Resource())
         return testing.TestClient(app)
 
@@ -266,15 +269,10 @@ def test_non_string_status(custom_status_client, status, expected_code):
 
 
 def test_deprecated_body():
-    with pytest.warns(DeprecatedWarning, match='Please use text instead'):
+    with pytest.raises(TypeError) as type_error:
         sts = HTTPStatus(falcon.HTTP_701, body='foo')
-        assert sts.text == 'foo'
-        assert sts.body == 'foo'
 
-        sts = HTTPStatus(falcon.HTTP_701, text='bar', body='foo')
-        assert sts.text == 'bar'
-        assert sts.body == 'bar'
+    assert 'unexpected keyword argument' in str(type_error.value)
 
-        sts = HTTPStatus(falcon.HTTP_701, text='', body='foo')
-        assert sts.text == ''
-        assert sts.body == ''
+    sts = HTTPStatus(falcon.HTTP_701, text='foo')
+    assert sts.text == 'foo'
