@@ -6,8 +6,6 @@ import falcon
 from falcon import media
 from falcon import testing
 
-from _util import create_app  # NOQA
-
 
 def test_deserialize_empty_form():
     handler = media.URLEncodedFormHandler()
@@ -49,8 +47,8 @@ class MediaMirrorAsync:
 
 
 @pytest.fixture
-def client(asgi):
-    app = create_app(asgi)
+def client(asgi, util):
+    app = util.create_app(asgi)
     app.add_route('/media', MediaMirrorAsync() if asgi else MediaMirror())
     return testing.TestClient(app)
 
