@@ -3,12 +3,10 @@ import pytest
 import falcon
 import falcon.testing as testing
 
-from _util import create_app  # NOQA
-
 
 @pytest.fixture
-def client(asgi):
-    app = create_app(asgi)
+def client(asgi, util):
+    app = util.create_app(asgi)
 
     resource = RedirectingResource()
     app.add_route('/', resource)
@@ -17,8 +15,8 @@ def client(asgi):
 
 
 @pytest.fixture
-def client_exercising_headers(asgi):
-    app = create_app(asgi)
+def client_exercising_headers(asgi, util):
+    app = util.create_app(asgi)
 
     resource = RedirectingResourceWithHeaders()
     app.add_route('/', resource)
