@@ -3,9 +3,16 @@ import json
 import pytest
 
 import falcon
-from falcon import errors, media, testing, util
+from falcon import errors
+from falcon import media
+from falcon import testing
+from falcon import util
+import falcon.asgi
 
-from _util import create_app  # NOQA
+try:
+    import msgpack
+except ImportError:
+    msgpack = None
 
 
 def create_client(asgi, handlers=None, resource=None):
@@ -14,7 +21,8 @@ def create_client(asgi, handlers=None, resource=None):
             testing.SimpleTestResourceAsync() if asgi else testing.SimpleTestResource()
         )
 
-    app = create_app(asgi)
+    app_cls = falcon.asgi.App if asgi else falcon.App
+    app = app_cls()
     app.add_route('/', resource)
 
     if handlers:
@@ -96,6 +104,7 @@ def test_json(client, media_type):
         ('application/x-msgpack'),
     ],
 )
+@pytest.mark.skipif(msgpack is None, reason='msgpack is required for this test')
 def test_msgpack(asgi, media_type):
     client = create_client(
         asgi,
@@ -148,6 +157,7 @@ def test_unknown_media_type(asgi, media_type):
 
 
 @pytest.mark.parametrize('media_type', ['application/json', 'application/msgpack'])
+@pytest.mark.skipif(msgpack is None, reason='msgpack is required for this test')
 def test_empty_body(asgi, media_type):
     client = _create_client_invalid_media(
         asgi,
@@ -184,13 +194,12 @@ def test_invalid_json(asgi):
     try:
         json.loads(expected_body)
     except Exception as e:
-        assert type(client.resource.captured_error.value.__cause__) == type(e)
+        assert type(client.resource.captured_error.value.__cause__) is type(e)
         assert str(client.resource.captured_error.value.__cause__) == str(e)
 
 
+@pytest.mark.skipif(msgpack is None, reason='msgpack is required for this test')
 def test_invalid_msgpack(asgi):
-    import msgpack
-
     handlers = {'application/msgpack': media.MessagePackHandler()}
     client = _create_client_invalid_media(
         asgi, errors.HTTPBadRequest, handlers=handlers
@@ -210,7 +219,7 @@ def test_invalid_msgpack(asgi):
     try:
         msgpack.unpackb(expected_body.encode('utf-8'))
     except Exception as e:
-        assert type(client.resource.captured_error.value.__cause__) == type(e)
+        assert type(client.resource.captured_error.value.__cause__) is type(e)
         assert str(client.resource.captured_error.value.__cause__) == str(e)
 
 
