@@ -6,6 +6,8 @@ path via simulate_get(), vs. probing the router directly.
 """
 
 from datetime import datetime
+from datetime import timezone
+import math
 import uuid
 
 import pytest
@@ -14,9 +16,6 @@ import falcon
 from falcon import testing
 from falcon.routing.util import SuffixedMethodNotFoundError
 
-from _util import create_app  # NOQA
-
-
 _TEST_UUID = uuid.uuid4()
 _TEST_UUID_2 = uuid.uuid4()
 _TEST_UUID_STR = str(_TEST_UUID)
@@ -24,6 +23,15 @@ _TEST_UUID_STR_2 = str(_TEST_UUID_2)
 _TEST_UUID_STR_SANS_HYPHENS = _TEST_UUID_STR.replace('-', '')
 
 
+def _as_params(*values, prefix=''):
+    # NOTE(caselit): each value must be a tuple/list even when using one
+    #   single argument
+    return [
+        pytest.param(*value, id=f'{prefix}_{i}' if prefix else f'{i}')
+        for i, value in enumerate(values, 1)
+    ]
+
+
 class IDResource:
     def __init__(self):
         self.id = None
@@ -134,8 +142,8 @@ def resource():
 
 
 @pytest.fixture
-def client(asgi):
-    return testing.TestClient(create_app(asgi))
+def client(asgi, util):
+    return testing.TestClient(util.create_app(asgi))
 
 
 def test_root_path(client, resource):
@@ -214,6 +222,49 @@ def test_int_converter(client, uri_template):
     assert resource1.req.path == '/123'
 
 
+@pytest.mark.parametrize('id_value', [2, 2.1, 1.9])
+@pytest.mark.parametrize(
+    'uri_template,',
+    [
+        '/{id:float}',
+        '/{id:float(1)}',
+        '/{id:float(min=1.9)}',
+        '/{id:float(min=1.8, max=3)}',
+    ],
+)
+def test_float_converter(client, uri_template, id_value):
+    resource1 = IDResource()
+    client.app.add_route(uri_template, resource1)
+
+    result = client.simulate_get('/{0}'.format(id_value))
+
+    assert result.status_code == 200
+    assert resource1.called
+    assert resource1.id == id_value
+    assert resource1.req.path == '/{0}'.format(id_value)
+
+
+@pytest.mark.parametrize('value', ['nan', '-inf', 'inf'])
+def test_float_converter_non_finite_allowed(value, client):
+    resource1 = IDResource()
+    client.app.add_route('/{id:float(finite=False)}', resource1)
+
+    result = client.simulate_get('/' + value)
+
+    assert result.status_code == 200
+    assert resource1.called
+    assert not math.isfinite(resource1.id)
+
+
+def test_float_converter_non_finite_disallowed(client):
+    resource1 = IDResource()
+    client.app.add_route('/{id:float}', resource1)
+
+    result = client.simulate_get('/NaN')
+    assert result.status_code == 404
+    assert not resource1.called
+
+
 @pytest.mark.parametrize(
     'uri_template,',
     [
@@ -238,7 +289,7 @@ def test_int_converter_rejections(client, uri_template):
         (
             '/{start_year:int}-to-{timestamp:dt}',
             '/1961-to-1969-07-21T02:56:00Z',
-            datetime(1969, 7, 21, 2, 56, 0),
+            datetime(1969, 7, 21, 2, 56, 0, tzinfo=timezone.utc),
         ),
         (
             '/{start_year:int}-to-{timestamp:dt("%Y-%m-%d")}',
@@ -270,7 +321,7 @@ def test_datetime_converter(client, resource, uri_template, path, dt_expected):
 
 @pytest.mark.parametrize(
     'uri_template, path, expected',
-    [
+    _as_params(
         (
             '/widgets/{widget_id:uuid}',
             '/widgets/' + _TEST_UUID_STR,
@@ -310,7 +361,8 @@ def test_datetime_converter(client, resource, uri_template, path, dt_expected):
             '/widgets/' + _TEST_UUID_STR_SANS_HYPHENS[:-1] + '/orders',
             None,
         ),
-    ],
+        prefix='uuid_converter',
+    ),
 )
 def test_uuid_converter(client, resource, uri_template, path, expected):
     client.app.add_route(uri_template, resource)
