@@ -6,7 +6,11 @@ import sys
 import time
 
 import pytest
-import requests
+
+try:
+    import requests
+except ImportError:
+    requests = None  # type: ignore
 
 from falcon import testing
 
@@ -28,6 +32,8 @@ def _gunicorn_args(host, port, extra_opts=()):
         pytest.skip('gunicorn not installed')
 
     args = (
+        sys.executable,
+        '-m',
         'gunicorn',
         '--access-logfile',
         '-',
@@ -72,6 +78,8 @@ def _uvicorn_args(host, port):
         pytest.skip('uvicorn not installed')
 
     return (
+        sys.executable,
+        '-m',
         'uvicorn',
         '--host',
         host,
@@ -102,7 +110,9 @@ def _waitress_args(host, port):
         pytest.skip('waitress not installed')
 
     return (
-        'waitress-serve',
+        sys.executable,
+        '-m',
+        'waitress',
         '--listen',
         '{}:{}'.format(host, port),
         '_wsgi_test_app:app',
@@ -184,6 +194,9 @@ def server_url(server_args):
         )
 
 
+@pytest.mark.skipif(
+    requests is None, reason='requests module is required for this test'
+)
 class TestWSGIServer:
     def test_get(self, server_url):
         resp = requests.get(server_url + '/hello', timeout=_REQUEST_TIMEOUT)
@@ -194,10 +207,9 @@ class TestWSGIServer:
 
     def test_get_deprecated(self, server_url):
         resp = requests.get(server_url + '/deprecated', timeout=_REQUEST_TIMEOUT)
-        assert resp.status_code == 200
-        assert resp.text == 'Hello, World!\n'
-        assert resp.headers.get('Content-Type') == 'text/plain; charset=utf-8'
-        assert resp.headers.get('X-Falcon') == 'deprecated'
+
+        # Since it tries to use resp.add_link() we expect an unhandled error
+        assert resp.status_code == 500
 
     def test_post_multipart_form(self, server_url):
         size = random.randint(8 * _SIZE_1_MB, 15 * _SIZE_1_MB)
