@@ -1,5 +1,7 @@
 #!/usr/bin/env bash
 
+set -e
+
 FALCON_ROOT=$( cd -- "$( dirname -- "${BASH_SOURCE[0]}" )/../.." &> /dev/null && pwd )
 MAILMAN_PATH=$FALCON_ROOT/.ecosystem/mailman
 
@@ -22,8 +24,14 @@ cd $MAILMAN_PATH
 cat <<EOT >> tox.ini
 
 [testenv:falcon-nocov]
-basepython = python3.8
+basepython = python3.12
 commands_pre =
     pip uninstall -y falcon
     pip install $FALCON_ROOT
 EOT
+
+# NOTE(vytas): The below test started failing on GitHub Actions:
+#   '=?utf-8?b?QSBtdWx0aWxpbmUgWy4uLl0=?=' != 'A multiline [...]'
+#   (but it works on other platforms).
+sed -i s/test_uheader_multiline/skip_test_uheader_multiline/ \
+    src/mailman/handlers/tests/test_cook_headers.py
